# Python
import unittest
from unittest.mock import Mock

# ATS
from pyats.topology import Device, loader

# Metaparser
from genie.metaparser.util.exceptions import SchemaEmptyParserError

from genie.libs.parser.junos.show_route import (ShowRouteTable,
                                                ShowRouteProtocolExtensive,
                                                ShowRouteInstanceDetail,
                                                ShowRoute,
                                                ShowRouteProtocolExtensive,
                                                ShowRouteSummary,
                                                ShowRouteAdvertisingProtocol,
                                                ShowRouteForwardingTableSummary,
                                                ShowRouteReceiveProtocol)

'''
Unit test for:
    * show route table {table}
    * show route table {table} {prefix}
'''
class test_show_route_table(unittest.TestCase):

    device = Device(name='aDevice')

    empty_output = {'execute.return_value': ''}

    golden_output_1 = {'execute.return_value': '''
        #show route table inet.3 10.64.4.4
     
        inet.3: 3 destinations, 3 routes (3 active, 0 holddown, 0 hidden)
        + = Active Route, - = Last Active, * = Both
         
        10.64.4.4/32         *[LDP/9] 03:40:50, metric 110
                            > to 192.168.220.6 via ge-0/0/1.0

    '''}

    parsed_output_1 = {
        'table_name': {
            'inet.3': {
                'active_route_count': 3,
                'destination_count': 3,
                'hidden_route_count': 0,
                'holddown_route_count': 0,
                'routes': {
                    '10.64.4.4/32': {
                        'active_tag': '*',
                        'age': '03:40:50',
                        'metric': '110',
                        'next_hop': {
                            'next_hop_list': {
                                1: {
                                    'best_route': '>',
                                    'to': '192.168.220.6',
                                    'via': 'ge-0/0/1.0'}}},
                        'preference': '9',
                        'protocol_name': 'LDP'}},
                'total_route_count': 3}}}

    golden_output_2 = {'execute.return_value': '''
        #show route table inet.3 192.168.36.220
     
        inet.3: 3 destinations, 3 routes (3 active, 0 holddown, 0 hidden)
        + = Active Route, - = Last Active, * = Both
         
        192.168.36.220/32 *[LDP/9] 03:41:19, metric 1111
                            > to 192.168.220.6 via ge-0/0/1.0, Push 305550

    '''}

    parsed_output_2 = {
        'table_name': {
            'inet.3': {
                'active_route_count': 3,
                'destination_count': 3,
                'hidden_route_count': 0,
                'holddown_route_count': 0,
                'routes': {
                    '192.168.36.220/32': {
                        'active_tag': '*',
                        'age': '03:41:19',
                        'metric': '1111',
                        'next_hop': {
                            'next_hop_list': {
                                1: {
                                    'best_route': '>',
                                    'mpls_label': 'Push 305550',
                                    'to': '192.168.220.6',
                                    'via': 'ge-0/0/1.0'}}},
                        'preference': '9',
                        'protocol_name': 'LDP'}},
                'total_route_count': 3}}}


    golden_output_3 = {'execute.return_value': '''
        #show route table inet.3

        inet.3: 3 destinations, 3 routes (3 active, 0 holddown, 0 hidden)
        + = Active Route, - = Last Active, * = Both

        10.64.4.4/32         *[LDP/9/4] 02:30:55, metric 110
                            > to 192.168.220.6 via ge-0/0/1.0
        10.169.197.254/32 *[LDP/9] 02:14:05, metric 1001
                            > to 10.49.0.1 via ge-0/0/2.0
        192.168.36.220/32 *[LDP/9] 02:03:22, metric 1111
                            > to 192.168.220.6 via ge-0/0/1.0, Push 307742
    '''}

    parsed_output_3 = {        
        'table_name': {
            'inet.3': {
                'active_route_count': 3,
                'destination_count': 3,
                'hidden_route_count': 0,
                'holddown_route_count': 0,
                'routes': {
                    '10.169.197.254/32': {
                        'active_tag': '*',
                        'age': '02:14:05',
                        'metric': '1001',
                        'next_hop': {
                            'next_hop_list': {
                                1: {
                                    'best_route': '>',
                                    'to': '10.49.0.1',
                                    'via': 'ge-0/0/2.0'}}},
                        'preference': '9',
                        'protocol_name': 'LDP'},
                    '192.168.36.220/32': {
                        'active_tag': '*',
                        'age': '02:03:22',
                        'metric': '1111',
                        'next_hop': {
                            'next_hop_list': {
                                1: {
                                    'best_route': '>',
                                    'mpls_label': 'Push 307742',
                                    'to': '192.168.220.6',
                                    'via': 'ge-0/0/1.0'}}},
                        'preference': '9',
                        'protocol_name': 'LDP'},
                    '10.64.4.4/32': {
                        'active_tag': '*',
                        'age': '02:30:55',
                        'metric': '110',
                        'next_hop': {
                            'next_hop_list': {
                                1: {
                                    'best_route': '>',
                                    'to': '192.168.220.6',
                                    'via': 'ge-0/0/1.0'}}},
                        'preference': '9',
                        'preference2': '4',
                        'protocol_name': 'LDP'}},
                        'total_route_count': 3}}}

    golden_output_4= {'execute.return_value': '''
        #show route table inet.3
        inet.3: 5 destinations, 5 routes (5 active, 0 holddown, 0 hidden)
        + = Active Route, - = Last Active, * = Both

        10.0.0.5/32        *[LDP/9] 00:25:43, metric 10
                              to 10.2.94.2 via lt-1/2/0.49
                            > to 10.2.3.2 via lt-1/2/0.23

    '''}

    parsed_output_4 = {
        'table_name': {
            'inet.3': {
                'active_route_count': 5,
                'destination_count': 5,
                'hidden_route_count': 0,
                'holddown_route_count': 0,
                'routes': {
                    '10.0.0.5/32': {
                        'active_tag': '*',
                        'age': '00:25:43',
                        'metric': '10',
                        'next_hop': {
                            'next_hop_list': {
                                1: {
                                    'to': '10.2.94.2',
                                    'via': 'lt-1/2/0.49'},
                                2: {
                                    'best_route': '>',
                                    'to': '10.2.3.2',
                                    'via': 'lt-1/2/0.23'}}},
                        'preference': '9',
                        'protocol_name': 'LDP'}},
                'total_route_count': 5}}}


    def test_show_route_table_empty(self):
        self.maxDiff = None
        self.device = Mock(**self.empty_output)
        obj = ShowRouteTable(device=self.device)
        with self.assertRaises(SchemaEmptyParserError):
            parsed_output = obj.parse(table='table.1')

    def test_show_route_table_1(self):
        self.maxDiff = None
        self.device = Mock(**self.golden_output_1)
        obj = ShowRouteTable(device=self.device)
        parsed_output = obj.parse(table='inet.3', prefix='10.64.4.4')
        self.assertEqual(parsed_output, self.parsed_output_1)

    def test_show_route_table_2(self):
        self.maxDiff = None
        self.device = Mock(**self.golden_output_2)
        obj = ShowRouteTable(device=self.device)
        parsed_output = obj.parse(table='inet.3', prefix='192.168.36.220')
        self.assertEqual(parsed_output, self.parsed_output_2)

    def test_show_route_table_3(self):
        self.maxDiff = None
        self.device = Mock(**self.golden_output_3)
        obj = ShowRouteTable(device=self.device)
        parsed_output = obj.parse(table='inet.3')
        self.assertEqual(parsed_output, self.parsed_output_3)

    def test_show_route_table_4(self):
        self.maxDiff = None
        self.device = Mock(**self.golden_output_4)
        obj = ShowRouteTable(device=self.device)
        parsed_output = obj.parse(table='inet.3')
        self.assertEqual(parsed_output, self.parsed_output_4)

'''
Unit test for:
    * show route
    * show route protocol {protocol}
    * show route protocol {protocol} {ip_address}
    * show route protocol {protocol} table {table}
'''
class TestShowRoute(unittest.TestCase):

    device = Device(name='aDevice')
    maxDiff = None

    empty_output = {'execute.return_value': ''}

    golden_output = {'execute.return_value': '''
        show route protocol static 10.169.14.240/32

        inet.0: 932 destinations, 1618 routes (932 active, 0 holddown, 0 hidden)
        + = Active Route, - = Last Active, * = Both

        10.169.14.240/32  *[Static/5] 5w2d 15:42:25
                            >  to 10.169.14.121 via ge-0/0/1.0

        inet.3: 12 destinations, 12 routes (12 active, 0 holddown, 0 hidden)
    '''}

    golden_parsed_output = {
        "route-information": {
            "route-table": [
                {
                    "active-route-count": "932",
                    "destination-count": "932",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [{
                        "rt-destination": "10.169.14.240/32",
                        "rt-entry": {
                            "active-tag": "*",
                            "age": {
                                "#text":"5w2d 15:42:25",
                            },
                            "nh": [{
                                "to": "10.169.14.121",
                                "via": "ge-0/0/1.0"
                            }],
                            "preference": "5",
                            "protocol-name": "Static"
                        }
                    }],
                    "table-name": "inet.0",
                    "total-route-count": "1618"
                },
                {
                    "active-route-count": "12",
                    "destination-count": "12",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "table-name": "inet.3",
                    "total-route-count": "12"
                }
            ]
        }
    }

    golden_output_2 = {'execute.return_value': '''
        show route protocol static 2001:db8:eb18:ca45::1

        inet6.0: 23 destinations, 24 routes (23 active, 0 holddown, 0 hidden)
        + = Active Route, - = Last Active, * = Both

        2001:db8:eb18:ca45::1/128
                        *[Static/5] 3w5d 18:30:36
                            >  to 2001:db8:eb18:6337::1 via ge-0/0/1.0
    '''}

    golden_parsed_output_2 = {
        "route-information": {
            "route-table": [
                {
                    "active-route-count": "23",
                    "destination-count": "23",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [{
                        "rt-destination": "2001:db8:eb18:ca45::1/128",
                        "rt-entry": {
                            "active-tag": "*",
                            "age": {
                                "#text": "3w5d 18:30:36",
                            },
                            "nh": [{
                                "to": "2001:db8:eb18:6337::1",
                                "via": "ge-0/0/1.0"
                            }],
                            "preference": "5",
                            "protocol-name": "Static"
                        }
                    }],
                    "table-name": "inet6.0",
                    "total-route-count": "24"
                }
            ]
        }
    }

    golden_output_3 = {'execute.return_value': '''
        show route protocol ospf

        inet.0: 929 destinations, 1615 routes (929 active, 0 holddown, 0 hidden)
        + = Active Route, - = Last Active, * = Both

        0.0.0.0/0          *[OSPF/150/10] 3w3d 03:24:58, metric 101, tag 0
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.1.0.0/24          [OSPF/150/10] 3w3d 03:24:58, metric 20, tag 0
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.36.3.3/32         *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.16.0.0/30         *[OSPF/10/10] 3w0d 04:51:49, metric 1200
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.100.5.5/32         *[OSPF/10/10] 3w0d 04:51:49, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.19.198.28/30    *[OSPF/10/10] 3w1d 17:03:19, metric 1005
                            >  to 10.189.5.94 via ge-0/0/0.0
        10.19.198.239/32   *[OSPF/10/10] 1w6d 20:52:58, metric 1001
                            >  to 10.19.198.26 via ge-0/0/2.0
        10.174.132.237/32   *[OSPF/150/10] 3w3d 03:24:58, metric 150, tag 0
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.34.2.200/30    *[OSPF/10/10] 3w0d 04:51:49, metric 205
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.34.2.250/32    *[OSPF/10/10] 3w0d 04:51:49, metric 200
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.34.2.251/32    *[OSPF/10/10] 3w0d 04:51:49, metric 205
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.15.0.0/30        *[OSPF/10/10] 1w6d 20:52:58, metric 1001
                            >  to 10.19.198.26 via ge-0/0/2.0
        10.64.0.0/30        *[OSPF/10/10] 1w0d 15:56:54, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.196.212/30 *[OSPF/10/10] 3w0d 04:51:49, metric 1200
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.196.216/30 *[OSPF/10/10] 3w0d 04:51:49, metric 1205
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.196.241/32 *[OSPF/10/10] 1w0d 15:56:54, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.16/30   *[OSPF/10/10] 3w3d 03:24:58, metric 105
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.32/30   *[OSPF/10/10] 3w0d 04:11:46, metric 225
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.128/30  *[OSPF/10/10] 3w1d 17:03:19, metric 125
                            >  to 10.189.5.94 via ge-0/0/0.0
        10.169.14.156/30  *[OSPF/10/10] 3w0d 04:51:51, metric 200
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.240/32   [OSPF/10/10] 3w3d 03:24:58, metric 100
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.241/32  *[OSPF/10/10] 3w3d 03:24:58, metric 105
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.242/32  *[OSPF/10/10] 3w3d 03:24:58, metric 100
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.243/32  *[OSPF/10/10] 3w3d 03:24:58, metric 105
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.189.5.253/32    *[OSPF/10/10] 3w1d 17:03:19, metric 5
                            >  to 10.189.5.94 via ge-0/0/0.0
        192.168.220.0/30       *[OSPF/10/10] 3w0d 04:51:49, metric 1200
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.0/32       *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.1/32       *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.2/32       *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.3/32       *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.4/32       *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.5/32       *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.6/32       *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.7/32       *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.8/32       *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.9/32       *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.10/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.11/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.12/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.13/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.14/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.15/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.16/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.17/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.18/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.19/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.20/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.21/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.22/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.23/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.24/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.25/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.26/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.27/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.28/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.29/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.30/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.31/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.32/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.33/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.34/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.35/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.36/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.37/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.38/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.39/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.40/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.41/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.42/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.43/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.44/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.45/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.46/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.47/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.48/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.49/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.50/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.51/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.52/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.53/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.54/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.55/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.56/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.57/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.58/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.59/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.60/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.61/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.62/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.63/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.64/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.65/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.66/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.67/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.68/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.69/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.70/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.71/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.72/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.73/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.74/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.75/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.76/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.77/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.78/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.79/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.80/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.81/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.82/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.83/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.84/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.85/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.86/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.87/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.88/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.89/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.90/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.91/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.92/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.93/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.94/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.95/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.96/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.97/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.98/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.99/32      *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.100/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.101/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.102/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.103/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.104/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.105/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.106/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.107/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.108/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.109/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.110/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.111/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.112/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.113/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.114/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.115/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.116/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.117/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.118/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.119/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.120/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.121/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.122/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.123/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.124/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.125/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.126/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.127/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.128/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.129/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.130/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.131/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.132/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.133/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.134/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.135/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.136/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.137/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.138/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.139/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.140/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.141/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.142/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.143/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.144/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.145/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.146/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.147/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.148/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.149/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.150/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.151/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.152/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.153/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.154/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.155/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.156/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.157/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.158/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.159/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.160/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.161/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.162/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.163/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.164/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.165/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.166/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.167/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.168/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.169/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.170/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.171/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.172/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.173/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.174/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.175/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.176/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.177/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.178/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.179/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.180/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.181/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.182/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.183/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.184/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.185/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.186/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.187/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.188/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.189/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.190/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.191/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.192/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.193/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.194/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.195/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.196/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.197/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.198/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.199/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.200/32     *[OSPF/10/10] 1w0d 15:56:54, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.111.0/30       *[OSPF/10/10] 1w0d 15:56:54, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.4.0/30       *[OSPF/10/10] 1w0d 15:56:29, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.100.0/25   *[OSPF/150/10] 2w6d 15:00:08, metric 32000, tag 65000500
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.100.252/32 *[OSPF/150/10] 2w6d 15:00:08, metric 32000, tag 65000500
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.36.48/30  *[OSPF/10/10] 3w3d 03:24:58, metric 10100
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.36.56/30  *[OSPF/10/10] 3w3d 03:24:58, metric 10100
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.36.119/32 *[OSPF/10/10] 3w3d 03:24:58, metric 10101
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.36.120/32 *[OSPF/10/10] 3w3d 03:24:58, metric 10101
                            >  to 10.169.14.121 via ge-0/0/1.0
        224.0.0.5/32       *[OSPF/10] 29w6d 21:48:09, metric 1
                            MultiRecv

        inet.3: 11 destinations, 11 routes (11 active, 0 holddown, 0 hidden)
        + = Active Route, - = Last Active, * = Both

        10.100.5.5/32         *[L-OSPF/10/5] 3w0d 04:11:34, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0, Push 17000
                            to 10.189.5.94 via ge-0/0/0.0, Push 17000, Push 1650, Push 1913(top)
        10.19.198.239/32   *[L-OSPF/10/5] 1w6d 20:52:49, metric 1001
                            >  to 10.19.198.26 via ge-0/0/2.0
                            to 10.189.5.94 via ge-0/0/0.0, Push 16073
        10.34.2.250/32    *[L-OSPF/10/5] 3w0d 04:51:49, metric 200
                            >  to 10.169.14.121 via ge-0/0/1.0, Push 16061
        10.34.2.251/32    *[L-OSPF/10/5] 3w0d 04:51:49, metric 205
                            >  to 10.169.14.121 via ge-0/0/1.0, Push 16062
        10.169.196.241/32 *[L-OSPF/10/5] 1w0d 15:56:49, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0, Push 16063
                            to 10.189.5.94 via ge-0/0/0.0, Push 16063, Push 1650, Push 1913(top)
        10.169.14.240/32  *[L-OSPF/10/5] 3w1d 17:03:14, metric 100
                            >  to 10.169.14.121 via ge-0/0/1.0
                            to 10.189.5.94 via ge-0/0/0.0, Push 16051, Push 1913(top)
        10.169.14.241/32  *[L-OSPF/10/5] 3w3d 03:24:58, metric 105
                            >  to 10.169.14.121 via ge-0/0/1.0, Push 16052
        10.189.5.253/32    *[L-OSPF/10/5] 3w1d 17:03:19, metric 5
                            >  to 10.189.5.94 via ge-0/0/0.0

        mpls.0: 44 destinations, 44 routes (44 active, 0 holddown, 0 hidden)
        + = Active Route, - = Last Active, * = Both

        2567               *[L-OSPF/10/5] 2w0d 19:07:25, metric 0
                            >  to 10.169.14.121 via ge-0/0/1.0, Pop      
                            to 10.189.5.94 via ge-0/0/0.0, Swap 16051, Push 1913(top)
        2567(S=0)          *[L-OSPF/10/5] 2w0d 19:07:25, metric 0
                            >  to 10.169.14.121 via ge-0/0/1.0, Pop      
                            to 10.189.5.94 via ge-0/0/0.0, Swap 16051, Push 1913(top)
        2568               *[L-OSPF/10/5] 3w3d 03:24:58, metric 0
                            >  to 10.169.14.121 via ge-0/0/1.0, Pop      
        2568(S=0)          *[L-OSPF/10/5] 3w3d 03:24:58, metric 0
                            >  to 10.169.14.121 via ge-0/0/1.0, Pop      
        16051              *[L-OSPF/10/5] 3w1d 17:03:14, metric 100
                            >  to 10.169.14.121 via ge-0/0/1.0, Pop      
                            to 10.189.5.94 via ge-0/0/0.0, Swap 16051, Push 1913(top)
        16051(S=0)         *[L-OSPF/10/5] 3w1d 17:03:14, metric 100
                            >  to 10.169.14.121 via ge-0/0/1.0, Pop      
                            to 10.189.5.94 via ge-0/0/0.0, Swap 16051, Push 1913(top)
        16052              *[L-OSPF/10/5] 3w3d 03:24:58, metric 105
                            >  to 10.169.14.121 via ge-0/0/1.0, Swap 16052
        16061              *[L-OSPF/10/5] 3w0d 04:51:49, metric 200
                            >  to 10.169.14.121 via ge-0/0/1.0, Swap 16061
        16062              *[L-OSPF/10/5] 3w0d 04:51:49, metric 205
                            >  to 10.169.14.121 via ge-0/0/1.0, Swap 16062
        16063              *[L-OSPF/10/5] 1w0d 15:56:49, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0, Swap 16063
                            to 10.189.5.94 via ge-0/0/0.0, Swap 16063, Push 1650, Push 1913(top)
        16072              *[L-OSPF/10/5] 3w1d 17:03:19, metric 5
                            >  to 10.189.5.94 via ge-0/0/0.0, Pop      
        16072(S=0)         *[L-OSPF/10/5] 3w1d 17:03:19, metric 5
                            >  to 10.189.5.94 via ge-0/0/0.0, Pop      
        16073              *[L-OSPF/10/5] 1w6d 20:52:49, metric 1001
                            >  to 10.19.198.26 via ge-0/0/2.0, Pop      
                            to 10.189.5.94 via ge-0/0/0.0, Swap 16073
        16073(S=0)         *[L-OSPF/10/5] 1w6d 20:52:49, metric 1001
                            >  to 10.19.198.26 via ge-0/0/2.0, Pop      
                            to 10.189.5.94 via ge-0/0/0.0, Swap 16073
        17000              *[L-OSPF/10/5] 3w0d 04:11:34, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0, Swap 17000
                            to 10.189.5.94 via ge-0/0/0.0, Swap 17000, Push 1650, Push 1913(top)
        28985              *[L-OSPF/10/5] 3w1d 17:03:19, metric 0
                            >  to 10.189.5.94 via ge-0/0/0.0, Pop      
        28985(S=0)         *[L-OSPF/10/5] 3w1d 17:03:19, metric 0
                            >  to 10.189.5.94 via ge-0/0/0.0, Pop      
        28986              *[L-OSPF/10/5] 3w1d 17:03:19, metric 0
                            >  to 10.189.5.94 via ge-0/0/0.0, Pop      
        28986(S=0)         *[L-OSPF/10/5] 3w1d 17:03:19, metric 0
                            >  to 10.189.5.94 via ge-0/0/0.0, Pop      
        167966             *[L-OSPF/10/5] 1w6d 20:52:49, metric 0
                            >  to 10.19.198.26 via ge-0/0/2.0, Pop      
                            to 10.189.5.94 via ge-0/0/0.0, Swap 16073
        167966(S=0)        *[L-OSPF/10/5] 1w6d 20:52:49, metric 0
                            >  to 10.19.198.26 via ge-0/0/2.0, Pop      
                            to 10.189.5.94 via ge-0/0/0.0, Swap 16073
        167967             *[L-OSPF/10/5] 1w6d 20:52:58, metric 0
                            >  to 10.19.198.26 via ge-0/0/2.0, Pop      
        167967(S=0)        *[L-OSPF/10/5] 1w6d 20:52:58, metric 0
                            >  to 10.19.198.26 via ge-0/0/2.0, Pop      

        inet6.0: 22 destinations, 23 routes (22 active, 0 holddown, 0 hidden)
        + = Active Route, - = Last Active, * = Both

        ::/0               *[OSPF3/150] 3w1d 17:03:18, metric 101, tag 0
                            >  to fe80::250:56ff:fe8d:72bd via ge-0/0/1.0
        2001:db8:6aa8:6a53::1001/128
                        *[OSPF3/150] 3w1d 17:03:18, metric 150, tag 0
                            >  to fe80::250:56ff:fe8d:72bd via ge-0/0/1.0
        2001:db8:b0f8:ca45::13/128
                        *[OSPF3/10] 3w0d 04:51:40, metric 200
                            >  to fe80::250:56ff:fe8d:72bd via ge-0/0/1.0
        2001:db8:b0f8:ca45::14/128
                        *[OSPF3/10] 3w0d 04:51:40, metric 205
                            >  to fe80::250:56ff:fe8d:72bd via ge-0/0/1.0
        2001:db8:b0f8:3ab::/64
                        *[OSPF3/10] 3w0d 04:51:40, metric 205
                            >  to fe80::250:56ff:fe8d:72bd via ge-0/0/1.0
        2001:db8:eb18:ca45::1/128
                            [OSPF3/10] 3w1d 17:03:18, metric 100
                            >  to fe80::250:56ff:fe8d:72bd via ge-0/0/1.0
        2001:db8:eb18:ca45::2/128
                        *[OSPF3/10] 3w1d 17:03:18, metric 105
                            >  to fe80::250:56ff:fe8d:72bd via ge-0/0/1.0
        2001:db8:eb18:e26e::/64
                        *[OSPF3/10] 3w1d 17:03:18, metric 105
                            >  to fe80::250:56ff:fe8d:72bd via ge-0/0/1.0
        2001:db8:eb18:f5e6::/64
                        *[OSPF3/10] 3w0d 04:11:35, metric 225
                            >  to fe80::250:56ff:fe8d:72bd via ge-0/0/1.0
        2001:db8:eb18:6d57::/64
                        *[OSPF3/10] 3w1d 17:03:23, metric 125
                            >  to fe80::250:56ff:fe8d:53c0 via ge-0/0/0.0
        2001:db8:eb18:9627::/64
                        *[OSPF3/10] 3w0d 04:51:40, metric 200
                            >  to fe80::250:56ff:fe8d:72bd via ge-0/0/1.0
        2001:db8:223c:ca45::c/128
                        *[OSPF3/10] 3w1d 17:03:23, metric 5
                            >  to fe80::250:56ff:fe8d:53c0 via ge-0/0/0.0
        ff02::5/128        *[OSPF3/10] 29w6d 21:48:09, metric 1
                            MultiRecv
    '''}

    golden_parsed_output_3 = {
        "route-information": {
            "route-table": [
                {
                    "active-route-count": "929",
                    "destination-count": "929",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [
                        {
                            "rt-destination": "0.0.0.0/0",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "101",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-tag": "0"
                            }
                        },
                        {
                            "rt-destination": "10.1.0.0/24",
                            "rt-entry": {
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "20",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-tag": "0"
                            }
                        },
                        {
                            "rt-destination": "10.36.3.3/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.16.0.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:49"
                                },
                                "metric": "1200",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.100.5.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:49"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.19.198.28/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:19"
                                },
                                "metric": "1005",
                                "nh": [
                                    {
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.19.198.239/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w6d 20:52:58"
                                },
                                "metric": "1001",
                                "nh": [
                                    {
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.174.132.237/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "150",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-tag": "0"
                            }
                        },
                        {
                            "rt-destination": "10.34.2.200/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:49"
                                },
                                "metric": "205",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.34.2.250/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:49"
                                },
                                "metric": "200",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.34.2.251/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:49"
                                },
                                "metric": "205",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.15.0.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w6d 20:52:58"
                                },
                                "metric": "1001",
                                "nh": [
                                    {
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.64.0.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.196.212/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:49"
                                },
                                "metric": "1200",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.196.216/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:49"
                                },
                                "metric": "1205",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.196.241/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.16/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "105",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.32/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:11:46"
                                },
                                "metric": "225",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.128/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:19"
                                },
                                "metric": "125",
                                "nh": [
                                    {
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.156/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:51"
                                },
                                "metric": "200",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.240/32",
                            "rt-entry": {
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "100",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.241/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "105",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.242/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "100",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.243/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "105",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.189.5.253/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:19"
                                },
                                "metric": "5",
                                "nh": [
                                    {
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:49"
                                },
                                "metric": "1200",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.0/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.1/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.2/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.3/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.4/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.6/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.7/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.8/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.9/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.10/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.11/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.12/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.13/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.14/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.15/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.16/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.17/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.18/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.19/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.20/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.21/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.22/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.23/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.24/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.25/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.26/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.27/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.28/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.29/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.30/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.31/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.32/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.33/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.34/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.35/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.36/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.37/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.38/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.39/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.40/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.41/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.42/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.43/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.44/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.45/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.46/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.47/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.48/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.49/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.50/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.51/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.52/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.53/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.54/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.55/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.56/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.57/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.58/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.59/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.60/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.61/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.62/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.63/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.64/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.65/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.66/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.67/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.68/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.69/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.70/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.71/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.72/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.73/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.74/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.75/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.76/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.77/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.78/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.79/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.80/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.81/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.82/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.83/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.84/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.85/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.86/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.87/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.88/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.89/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.90/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.91/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.92/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.93/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.94/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.95/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.96/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.97/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.98/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.99/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.100/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.101/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.102/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.103/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.104/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.105/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.106/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.107/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.108/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.109/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.110/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.111/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.112/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.113/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.114/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.115/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.116/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.117/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.118/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.119/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.120/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.121/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.122/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.123/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.124/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.125/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.126/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.127/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.128/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.129/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.130/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.131/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.132/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.133/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.134/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.135/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.136/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.137/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.138/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.139/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.140/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.141/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.142/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.143/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.144/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.145/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.146/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.147/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.148/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.149/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.150/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.151/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.152/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.153/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.154/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.155/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.156/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.157/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.158/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.159/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.160/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.161/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.162/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.163/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.164/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.165/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.166/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.167/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.168/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.169/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.170/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.171/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.172/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.173/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.174/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.175/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.176/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.177/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.178/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.179/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.180/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.181/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.182/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.183/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.184/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.185/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.186/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.187/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.188/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.189/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.190/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.191/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.192/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.193/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.194/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.195/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.196/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.197/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.198/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.199/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.200/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.111.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:54"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.4.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:29"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.100.0/25",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 15:00:08"
                                },
                                "metric": "32000",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-tag": "65000500"
                            }
                        },
                        {
                            "rt-destination": "192.168.100.252/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 15:00:08"
                                },
                                "metric": "32000",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-tag": "65000500"
                            }
                        },
                        {
                            "rt-destination": "192.168.36.48/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "10100",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.36.56/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "10100",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.36.119/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "10101",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.36.120/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "10101",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "224.0.0.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "29w6d 21:48:09"
                                },
                                "metric": "1",
                                "nh-type": "MultiRecv",
                                "preference": "10",
                                "protocol-name": "OSPF"
                            }
                        }
                    ],
                    "table-name": "inet.0",
                    "total-route-count": "1615"
                },
                {
                    "active-route-count": "11",
                    "destination-count": "11",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [
                        {
                            "rt-destination": "10.100.5.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:11:34"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "mpls-label": "Push 17000",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    },
                                    {
                                        "mpls-label": "Push 17000, Push 1650, Push 1913(top)",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.19.198.239/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w6d 20:52:49"
                                },
                                "metric": "1001",
                                "nh": [
                                    {
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0"
                                    },
                                    {
                                        "mpls-label": "Push 16073",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.34.2.250/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:49"
                                },
                                "metric": "200",
                                "nh": [
                                    {
                                        "mpls-label": "Push 16061",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.34.2.251/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:49"
                                },
                                "metric": "205",
                                "nh": [
                                    {
                                        "mpls-label": "Push 16062",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.196.241/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:49"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "mpls-label": "Push 16063",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    },
                                    {
                                        "mpls-label": "Push 16063, Push 1650, Push 1913(top)",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.240/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:14"
                                },
                                "metric": "100",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    },
                                    {
                                        "mpls-label": "Push 16051, Push 1913(top)",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.241/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "105",
                                "nh": [
                                    {
                                        "mpls-label": "Push 16052",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.189.5.253/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:19"
                                },
                                "metric": "5",
                                "nh": [
                                    {
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        }
                    ],
                    "table-name": "inet.3",
                    "total-route-count": "11"
                },
                {
                    "active-route-count": "44",
                    "destination-count": "44",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [
                        {
                            "rt-destination": "2567",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w0d 19:07:25"
                                },
                                "metric": "0",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    },
                                    {
                                        "mpls-label": "Swap 16051, Push 1913(top)",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "2567(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w0d 19:07:25"
                                },
                                "metric": "0",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    },
                                    {
                                        "mpls-label": "Swap 16051, Push 1913(top)",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "2568",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "0",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "2568(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "0",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "16051",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:14"
                                },
                                "metric": "100",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    },
                                    {
                                        "mpls-label": "Swap 16051, Push 1913(top)",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "16051(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:14"
                                },
                                "metric": "100",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    },
                                    {
                                        "mpls-label": "Swap 16051, Push 1913(top)",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "16052",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:24:58"
                                },
                                "metric": "105",
                                "nh": [
                                    {
                                        "mpls-label": "Swap 16052",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "16061",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:49"
                                },
                                "metric": "200",
                                "nh": [
                                    {
                                        "mpls-label": "Swap 16061",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "16062",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:49"
                                },
                                "metric": "205",
                                "nh": [
                                    {
                                        "mpls-label": "Swap 16062",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "16063",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:56:49"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "mpls-label": "Swap 16063",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    },
                                    {
                                        "mpls-label": "Swap 16063, Push 1650, Push 1913(top)",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "16072",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:19"
                                },
                                "metric": "5",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "16072(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:19"
                                },
                                "metric": "5",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "16073",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w6d 20:52:49"
                                },
                                "metric": "1001",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0"
                                    },
                                    {
                                        "mpls-label": "Swap 16073",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "16073(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w6d 20:52:49"
                                },
                                "metric": "1001",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0"
                                    },
                                    {
                                        "mpls-label": "Swap 16073",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "17000",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:11:34"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "mpls-label": "Swap 17000",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    },
                                    {
                                        "mpls-label": "Swap 17000, Push 1650, Push 1913(top)",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "28985",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:19"
                                },
                                "metric": "0",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "28985(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:19"
                                },
                                "metric": "0",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "28986",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:19"
                                },
                                "metric": "0",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "28986(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:19"
                                },
                                "metric": "0",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "167966",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w6d 20:52:49"
                                },
                                "metric": "0",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0"
                                    },
                                    {
                                        "mpls-label": "Swap 16073",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "167966(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w6d 20:52:49"
                                },
                                "metric": "0",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0"
                                    },
                                    {
                                        "mpls-label": "Swap 16073",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "167967",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w6d 20:52:58"
                                },
                                "metric": "0",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        },
                        {
                            "rt-destination": "167967(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w6d 20:52:58"
                                },
                                "metric": "0",
                                "nh": [
                                    {
                                        "mpls-label": "Pop",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF"
                            }
                        }
                    ],
                    "table-name": "mpls.0",
                    "total-route-count": "44"
                },
                {
                    "active-route-count": "22",
                    "destination-count": "22",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [
                        {
                            "rt-destination": "::/0",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:18"
                                },
                                "metric": "101",
                                "nh": [
                                    {
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "protocol-name": "OSPF3",
                                "rt-tag": "0"
                            }
                        },
                        {
                            "rt-destination": "2001:db8:6aa8:6a53::1001/128",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:18"
                                },
                                "metric": "150",
                                "nh": [
                                    {
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "protocol-name": "OSPF3",
                                "rt-tag": "0"
                            }
                        },
                        {
                            "rt-destination": "2001:db8:b0f8:ca45::13/128",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:40"
                                },
                                "metric": "200",
                                "nh": [
                                    {
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "protocol-name": "OSPF3"
                            }
                        },
                        {
                            "rt-destination": "2001:db8:b0f8:ca45::14/128",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:40"
                                },
                                "metric": "205",
                                "nh": [
                                    {
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "protocol-name": "OSPF3"
                            }
                        },
                        {
                            "rt-destination": "2001:db8:b0f8:3ab::/64",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:40"
                                },
                                "metric": "205",
                                "nh": [
                                    {
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "protocol-name": "OSPF3"
                            }
                        },
                        {
                            "rt-destination": "2001:db8:eb18:ca45::1/128",
                            "rt-entry": {
                                "age": {
                                    "#text": "3w1d 17:03:18"
                                },
                                "metric": "100",
                                "nh": [
                                    {
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "protocol-name": "OSPF3"
                            }
                        },
                        {
                            "rt-destination": "2001:db8:eb18:ca45::2/128",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:18"
                                },
                                "metric": "105",
                                "nh": [
                                    {
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "protocol-name": "OSPF3"
                            }
                        },
                        {
                            "rt-destination": "2001:db8:eb18:e26e::/64",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:18"
                                },
                                "metric": "105",
                                "nh": [
                                    {
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "protocol-name": "OSPF3"
                            }
                        },
                        {
                            "rt-destination": "2001:db8:eb18:f5e6::/64",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:11:35"
                                },
                                "metric": "225",
                                "nh": [
                                    {
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "protocol-name": "OSPF3"
                            }
                        },
                        {
                            "rt-destination": "2001:db8:eb18:6d57::/64",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:23"
                                },
                                "metric": "125",
                                "nh": [
                                    {
                                        "to": "fe80::250:56ff:fe8d:53c0",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "protocol-name": "OSPF3"
                            }
                        },
                        {
                            "rt-destination": "2001:db8:eb18:9627::/64",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:51:40"
                                },
                                "metric": "200",
                                "nh": [
                                    {
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "protocol-name": "OSPF3"
                            }
                        },
                        {
                            "rt-destination": "2001:db8:223c:ca45::c/128",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w1d 17:03:23"
                                },
                                "metric": "5",
                                "nh": [
                                    {
                                        "to": "fe80::250:56ff:fe8d:53c0",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "protocol-name": "OSPF3"
                            }
                        },
                        {
                            "rt-destination": "ff02::5/128",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "29w6d 21:48:09"
                                },
                                "metric": "1",
                                "nh-type": "MultiRecv",
                                "preference": "10",
                                "protocol-name": "OSPF3"
                            }
                        }
                    ],
                    "table-name": "inet6.0",
                    "total-route-count": "23"
                }
            ]
        }
    }

    golden_output_4 = {'execute.return_value': '''
        show route protocol ospf table inet.0

        inet.0: 929 destinations, 1615 routes (929 active, 0 holddown, 0 hidden)
        + = Active Route, - = Last Active, * = Both

        0.0.0.0/0          *[OSPF/150/10] 3w2d 04:45:37, metric 101, tag 0
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.1.0.0/24          [OSPF/150/10] 3w2d 04:45:37, metric 20, tag 0
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.36.3.3/32         *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.16.0.0/30         *[OSPF/10/10] 2w6d 06:12:28, metric 1200
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.100.5.5/32         *[OSPF/10/10] 2w6d 06:12:28, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.19.198.28/30    *[OSPF/10/10] 3w0d 18:23:58, metric 1005
                            >  to 10.189.5.94 via ge-0/0/0.0
        10.19.198.239/32   *[OSPF/10/10] 1w5d 22:13:37, metric 1001
                            >  to 10.19.198.26 via ge-0/0/2.0
        10.174.132.237/32   *[OSPF/150/10] 3w2d 04:45:37, metric 150, tag 0
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.34.2.200/30    *[OSPF/10/10] 2w6d 06:12:28, metric 205
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.34.2.250/32    *[OSPF/10/10] 2w6d 06:12:28, metric 200
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.34.2.251/32    *[OSPF/10/10] 2w6d 06:12:28, metric 205
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.15.0.0/30        *[OSPF/10/10] 1w5d 22:13:37, metric 1001
                            >  to 10.19.198.26 via ge-0/0/2.0
        10.64.0.0/30        *[OSPF/10/10] 6d 17:17:33, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.196.212/30 *[OSPF/10/10] 2w6d 06:12:28, metric 1200
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.196.216/30 *[OSPF/10/10] 2w6d 06:12:28, metric 1205
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.196.241/32 *[OSPF/10/10] 6d 17:17:33, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.16/30   *[OSPF/10/10] 3w2d 04:45:37, metric 105
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.32/30   *[OSPF/10/10] 2w6d 05:32:25, metric 225
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.128/30  *[OSPF/10/10] 3w0d 18:23:58, metric 125
                            >  to 10.189.5.94 via ge-0/0/0.0
        10.169.14.156/30  *[OSPF/10/10] 2w6d 06:12:30, metric 200
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.240/32   [OSPF/10/10] 3w2d 04:45:37, metric 100
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.241/32  *[OSPF/10/10] 3w2d 04:45:37, metric 105
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.242/32  *[OSPF/10/10] 3w2d 04:45:37, metric 100
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.169.14.243/32  *[OSPF/10/10] 3w2d 04:45:37, metric 105
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.189.5.253/32    *[OSPF/10/10] 3w0d 18:23:58, metric 5
                            >  to 10.189.5.94 via ge-0/0/0.0
        192.168.220.0/30       *[OSPF/10/10] 2w6d 06:12:28, metric 1200
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.0/32       *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.1/32       *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.2/32       *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.3/32       *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.4/32       *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.5/32       *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.6/32       *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.7/32       *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.8/32       *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.9/32       *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.10/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.11/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.12/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.13/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.14/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.15/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.16/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.17/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.18/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.19/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.20/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.21/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.22/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.23/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.24/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.25/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.26/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.27/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.28/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.29/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.30/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.31/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.32/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.33/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.34/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.35/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.36/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.37/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.38/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.39/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.40/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.41/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.42/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.43/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.44/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.45/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.46/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.47/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.48/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.49/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.50/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.51/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.52/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.53/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.54/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.55/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.56/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.57/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.58/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.59/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.60/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.61/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.62/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.63/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.64/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.65/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.66/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.67/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.68/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.69/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.70/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.71/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.72/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.73/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.74/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.75/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.76/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.77/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.78/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.79/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.80/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.81/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.82/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.83/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.84/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.85/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.86/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.87/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.88/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.89/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.90/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.91/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.92/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.93/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.94/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.95/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.96/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.97/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.98/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.99/32      *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.100/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.101/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.102/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.103/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.104/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.105/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.106/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.107/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.108/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.109/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.110/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.111/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.112/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.113/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.114/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.115/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.116/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.117/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.118/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.119/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.120/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.121/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.122/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.123/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.124/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.125/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.126/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.127/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.128/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.129/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.130/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.131/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.132/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.133/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.134/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.135/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.136/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.137/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.138/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.139/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.140/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.141/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.142/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.143/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.144/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.145/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.146/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.147/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.148/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.149/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.150/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.151/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.152/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.153/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.154/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.155/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.156/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.157/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.158/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.159/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.160/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.161/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.162/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.163/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.164/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.165/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.166/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.167/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.168/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.169/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.170/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.171/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.172/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.173/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.174/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.175/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.176/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.177/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.178/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.179/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.180/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.181/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.182/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.183/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.184/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.185/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.186/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.187/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.188/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.189/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.190/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.191/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.192/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.193/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.194/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.195/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.196/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.197/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.198/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.199/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.220.200/32     *[OSPF/10/10] 6d 17:17:33, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.111.0/30       *[OSPF/10/10] 6d 17:17:33, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.4.0/30       *[OSPF/10/10] 6d 17:17:08, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.100.0/25   *[OSPF/150/10] 2w5d 16:20:47, metric 32000, tag 65000500
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.100.252/32 *[OSPF/150/10] 2w5d 16:20:47, metric 32000, tag 65000500
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.36.48/30  *[OSPF/10/10] 3w2d 04:45:37, metric 10100
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.36.56/30  *[OSPF/10/10] 3w2d 04:45:37, metric 10100
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.36.119/32 *[OSPF/10/10] 3w2d 04:45:37, metric 10101
                            >  to 10.169.14.121 via ge-0/0/1.0
        192.168.36.120/32 *[OSPF/10/10] 3w2d 04:45:37, metric 10101
                            >  to 10.169.14.121 via ge-0/0/1.0
        224.0.0.5/32       *[OSPF/10] 29w5d 23:08:48, metric 1
                            MultiRecv
    '''}

    golden_parsed_output_4 = {
        "route-information": {
            "route-table": [
                {
                    "active-route-count": "929",
                    "destination-count": "929",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [
                        {
                            "rt-destination": "0.0.0.0/0",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 04:45:37"
                                },
                                "metric": "101",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-tag": "0"
                            }
                        },
                        {
                            "rt-destination": "10.1.0.0/24",
                            "rt-entry": {
                                "age": {
                                    "#text": "3w2d 04:45:37"
                                },
                                "metric": "20",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-tag": "0"
                            }
                        },
                        {
                            "rt-destination": "10.36.3.3/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.16.0.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 06:12:28"
                                },
                                "metric": "1200",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.100.5.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 06:12:28"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.19.198.28/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:23:58"
                                },
                                "metric": "1005",
                                "nh": [
                                    {
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.19.198.239/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w5d 22:13:37"
                                },
                                "metric": "1001",
                                "nh": [
                                    {
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.174.132.237/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 04:45:37"
                                },
                                "metric": "150",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-tag": "0"
                            }
                        },
                        {
                            "rt-destination": "10.34.2.200/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 06:12:28"
                                },
                                "metric": "205",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.34.2.250/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 06:12:28"
                                },
                                "metric": "200",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.34.2.251/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 06:12:28"
                                },
                                "metric": "205",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.15.0.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w5d 22:13:37"
                                },
                                "metric": "1001",
                                "nh": [
                                    {
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.64.0.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.196.212/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 06:12:28"
                                },
                                "metric": "1200",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.196.216/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 06:12:28"
                                },
                                "metric": "1205",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.196.241/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.16/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 04:45:37"
                                },
                                "metric": "105",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.32/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 05:32:25"
                                },
                                "metric": "225",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.128/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:23:58"
                                },
                                "metric": "125",
                                "nh": [
                                    {
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.156/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 06:12:30"
                                },
                                "metric": "200",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.240/32",
                            "rt-entry": {
                                "age": {
                                    "#text": "3w2d 04:45:37"
                                },
                                "metric": "100",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.241/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 04:45:37"
                                },
                                "metric": "105",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.242/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 04:45:37"
                                },
                                "metric": "100",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.169.14.243/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 04:45:37"
                                },
                                "metric": "105",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.189.5.253/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:23:58"
                                },
                                "metric": "5",
                                "nh": [
                                    {
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 06:12:28"
                                },
                                "metric": "1200",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.0/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.1/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.2/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.3/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.4/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.6/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.7/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.8/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.9/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.10/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.11/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.12/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.13/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.14/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.15/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.16/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.17/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.18/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.19/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.20/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.21/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.22/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.23/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.24/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.25/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.26/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.27/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.28/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.29/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.30/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.31/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.32/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.33/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.34/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.35/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.36/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.37/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.38/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.39/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.40/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.41/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.42/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.43/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.44/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.45/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.46/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.47/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.48/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.49/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.50/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.51/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.52/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.53/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.54/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.55/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.56/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.57/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.58/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.59/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.60/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.61/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.62/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.63/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.64/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.65/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.66/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.67/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.68/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.69/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.70/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.71/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.72/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.73/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.74/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.75/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.76/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.77/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.78/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.79/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.80/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.81/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.82/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.83/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.84/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.85/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.86/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.87/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.88/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.89/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.90/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.91/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.92/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.93/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.94/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.95/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.96/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.97/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.98/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.99/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.100/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.101/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.102/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.103/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.104/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.105/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.106/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.107/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.108/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.109/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.110/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.111/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.112/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.113/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.114/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.115/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.116/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.117/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.118/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.119/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.120/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.121/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.122/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.123/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.124/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.125/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.126/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.127/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.128/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.129/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.130/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.131/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.132/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.133/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.134/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.135/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.136/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.137/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.138/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.139/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.140/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.141/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.142/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.143/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.144/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.145/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.146/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.147/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.148/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.149/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.150/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.151/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.152/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.153/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.154/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.155/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.156/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.157/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.158/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.159/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.160/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.161/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.162/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.163/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.164/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.165/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.166/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.167/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.168/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.169/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.170/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.171/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.172/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.173/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.174/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.175/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.176/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.177/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.178/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.179/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.180/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.181/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.182/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.183/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.184/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.185/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.186/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.187/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.188/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.189/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.190/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.191/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.192/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.193/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.194/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.195/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.196/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.197/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.198/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.199/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.220.200/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.111.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:33"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.4.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:17:08"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.100.0/25",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w5d 16:20:47"
                                },
                                "metric": "32000",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-tag": "65000500"
                            }
                        },
                        {
                            "rt-destination": "192.168.100.252/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w5d 16:20:47"
                                },
                                "metric": "32000",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-tag": "65000500"
                            }
                        },
                        {
                            "rt-destination": "192.168.36.48/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 04:45:37"
                                },
                                "metric": "10100",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.36.56/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 04:45:37"
                                },
                                "metric": "10100",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.36.119/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 04:45:37"
                                },
                                "metric": "10101",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "192.168.36.120/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 04:45:37"
                                },
                                "metric": "10101",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "224.0.0.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "29w5d 23:08:48"
                                },
                                "metric": "1",
                                "nh-type": "MultiRecv",
                                "preference": "10",
                                "protocol-name": "OSPF"
                            }
                        }
                    ],
                    "table-name": "inet.0",
                    "total-route-count": "1615"
                }
            ]
        }
    }

    golden_output_5 = {'execute.return_value': '''
        show route 

        inet.0: 929 destinations, 1615 routes (929 active, 0 holddown, 0 hidden)
        + = Active Route, - = Last Active, * = Both

        0.0.0.0/0          *[OSPF/150/10] 3w3d 03:12:45, metric 101, tag 0
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.1.0.0/24         *[Direct/0] 29w6d 21:35:55
                            >  via fxp0.0
                            [OSPF/150/10] 3w3d 03:12:45, metric 20, tag 0
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.1.0.101/32       *[Local/0] 29w6d 21:35:55
                            Local via fxp0.0
        10.36.3.3/32         *[OSPF/10/10] 1w0d 15:44:41, metric 1202
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.16.0.0/30         *[OSPF/10/10] 3w0d 04:39:36, metric 1200
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.100.5.5/32         *[OSPF/10/10] 3w0d 04:39:36, metric 1201
                            >  to 10.169.14.121 via ge-0/0/1.0
        10.220.0.0/16      *[BGP/170] 3w3d 03:12:24, MED 12003, localpref 120, from 10.169.14.240
                            AS path: (65151 65000) I, validation-state: unverified
                            >  to 10.169.14.121 via ge-0/0/1.0
                            [BGP/170] 3w1d 16:51:06, MED 12003, localpref 120, from 10.189.5.253
                            AS path: (65151 65000) I, validation-state: unverified
                            >  to 10.189.5.94 via ge-0/0/0.0
    '''}

    golden_parsed_output_5 = {
        "route-information": {
            "route-table": [
                {
                    "active-route-count": "929",
                    "destination-count": "929",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [
                        {
                            "rt-destination": "0.0.0.0/0",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:12:45"
                                },
                                "metric": "101",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-tag": "0"
                            }
                        },
                        {
                            "rt-destination": "10.1.0.0/24",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "29w6d 21:35:55"
                                },
                                "nh": [
                                    {
                                        "via": "fxp0.0"
                                    }
                                ],
                                "preference": "0",
                                "protocol-name": "Direct"
                            }
                        },
                        {
                            "rt-entry": {
                                "age": {
                                    "#text": "3w3d 03:12:45"
                                },
                                "metric": "20",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-tag": "0"
                            }
                        },
                        {
                            "rt-destination": "10.1.0.101/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "29w6d 21:35:55"
                                },
                                "nh": [
                                    {
                                        "nh-local-interface": "fxp0.0"
                                    }
                                ],
                                "preference": "0",
                                "protocol-name": "Local"
                            }
                        },
                        {
                            "rt-destination": "10.36.3.3/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w0d 15:44:41"
                                },
                                "metric": "1202",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.16.0.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:39:36"
                                },
                                "metric": "1200",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.100.5.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 04:39:36"
                                },
                                "metric": "1201",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF"
                            }
                        },
                        {
                            "rt-destination": "10.220.0.0/16",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w3d 03:12:24"
                                },
                                "as-path": " (65151 65000) I",
                                "learned-from": "10.169.14.240",
                                "local-preference": "120",
                                "med": "12003",
                                "nh": [
                                    {
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "preference": "170",
                                "protocol-name": "BGP",
                                "validation-state": "unverified"
                            }
                        },
                        {
                            "rt-entry": {
                                "age": {
                                    "#text": "3w1d 16:51:06"
                                },
                                "as-path": " (65151 65000) I",
                                "learned-from": "10.189.5.253",
                                "local-preference": "120",
                                "med": "12003",
                                "nh": [
                                    {
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "preference": "170",
                                "protocol-name": "BGP",
                                "validation-state": "unverified"
                            }
                        }
                    ],
                    "table-name": "inet.0",
                    "total-route-count": "1615"
                }
            ]
        }
    }

    def test_empty(self):
        self.device = Mock(**self.empty_output)
        obj = ShowRoute(device=self.device)
        with self.assertRaises(SchemaEmptyParserError):
            parsed_output = obj.parse(
                protocol='static',
                ip_address='10.169.14.240/32')

    def test_golden(self):
        self.device = Mock(**self.golden_output)
        obj = ShowRoute(device=self.device)
        parsed_output = obj.parse(
            protocol='static',
            ip_address='10.169.14.240/32')
        self.assertEqual(parsed_output, self.golden_parsed_output)
    
    def test_golden_2(self):
        self.device = Mock(**self.golden_output_2)
        obj = ShowRoute(device=self.device)
        parsed_output = obj.parse(
            protocol='static',
            ip_address='2001:db8:eb18:ca45::1')
        self.assertEqual(parsed_output, self.golden_parsed_output_2)
    
    def test_golden_3(self):
        self.device = Mock(**self.golden_output_3)
        obj = ShowRoute(device=self.device)
        parsed_output = obj.parse(
            protocol='ospf')
        self.assertEqual(parsed_output, self.golden_parsed_output_3)

    def test_golden_4(self):
        self.device = Mock(**self.golden_output_4)
        obj = ShowRoute(device=self.device)
        parsed_output = obj.parse(
            protocol='ospf',
            table='inet.0')
        self.assertEqual(parsed_output, self.golden_parsed_output_4)
    
    def test_golden_5(self):
        self.device = Mock(**self.golden_output_5)
        obj = ShowRoute(device=self.device)
        parsed_output = obj.parse()
        self.assertEqual(parsed_output, self.golden_parsed_output_5)

'''
Unit test for:
    * show route protocol {protocol} extensive
    * show route protocol {protocol} table {table} extensive
'''
class TestShowRouteProtocolExtensive(unittest.TestCase):

    device = Device(name='aDevice')
    maxDiff = None

    empty_output = {'execute.return_value': ''}

    golden_output = {'execute.return_value': '''
        show route protocol ospf extensive

        inet.0: 929 destinations, 1615 routes (929 active, 0 holddown, 0 hidden)
        0.0.0.0/0 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 0.0.0.0/0 -> {10.169.14.121}
                *OSPF   Preference: 150/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int Ext>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 101 
                        Validation State: unverified 
                            Tag: 0 
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.1.0.0/24 (2 entries, 1 announced)
                State: <FlashAll>
                OSPF   Preference: 150/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Int Ext>
                        Inactive reason: Route Preference
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 20 
                        Validation State: unverified 
                            Tag: 0 
                        Task: OSPF
                        AS path: I 

        10.36.3.3/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.36.3.3/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.16.0.0/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.16.0.0/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:26 	Metric: 1200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.100.5.5/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.100.5.5/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:26 	Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        10.19.198.28/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.19.198.28/30 -> {10.189.5.94}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 614
                        Address: 0xdbe48d4
                        Next-hop reference count: 6
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0x1, selected
                        Session Id: 0x140
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:56 	Metric: 1005 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.19.198.239/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.19.198.239/32 -> {10.19.198.26}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 605
                        Address: 0xdbc0d54
                        Next-hop reference count: 4
                        Next hop: 10.19.198.26 via ge-0/0/2.0 weight 0x1, selected
                        Session Id: 0x1ae
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 1w5d 22:11:35 	Metric: 1001 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        10.174.132.237/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.174.132.237/32 -> {10.169.14.121}
                *OSPF   Preference: 150/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int Ext>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 150 
                        Validation State: unverified 
                            Tag: 0 
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.34.2.200/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.34.2.200/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:26 	Metric: 205 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.34.2.250/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.34.2.250/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:26 	Metric: 200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        10.34.2.251/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.34.2.251/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:26 	Metric: 205 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        10.15.0.0/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.15.0.0/30 -> {10.19.198.26}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 605
                        Address: 0xdbc0d54
                        Next-hop reference count: 4
                        Next hop: 10.19.198.26 via ge-0/0/2.0 weight 0x1, selected
                        Session Id: 0x1ae
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 1w5d 22:11:35 	Metric: 1001 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.64.0.0/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.64.0.0/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.196.212/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.196.212/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:26 	Metric: 1200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.196.216/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.196.216/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:26 	Metric: 1205 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.196.241/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.196.241/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        10.169.14.16/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.16/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 105 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.14.32/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.32/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 5:30:23 	Metric: 225 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.14.128/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.128/30 -> {10.189.5.94}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 614
                        Address: 0xdbe48d4
                        Next-hop reference count: 6
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0x1, selected
                        Session Id: 0x140
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:56 	Metric: 125 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.14.156/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.156/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:28 	Metric: 200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.14.240/32 (2 entries, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.240/32 -> {10.169.14.121}
        OSPF area : 0.0.0.0, LSA ID : 10.169.14.240, LSA type : Extern
                OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Int>
                        Inactive reason: Route Preference
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 100 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        AS path: I 

        10.169.14.241/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.241/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 105 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        10.169.14.242/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.242/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 100 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.14.243/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.243/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 105 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.189.5.253/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.189.5.253/32 -> {10.189.5.94}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 614
                        Address: 0xdbe48d4
                        Next-hop reference count: 6
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0x1, selected
                        Session Id: 0x140
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:56 	Metric: 5 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        192.168.220.0/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.0/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:26 	Metric: 1200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.0/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.0/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.1/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.1/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.2/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.2/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.3/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.3/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.4/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.4/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.5/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.5/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.6/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.6/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.7/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.7/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.8/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.8/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.9/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.9/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.10/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.10/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.11/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.11/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.12/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.12/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.13/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.13/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.14/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.14/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.15/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.15/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.16/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.16/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.17/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.17/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.18/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.18/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.19/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.19/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.20/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.20/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.21/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.21/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.22/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.22/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.23/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.23/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.24/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.24/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.25/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.25/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.26/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.26/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.27/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.27/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.28/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.28/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.29/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.29/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.30/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.30/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.31/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.31/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.32/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.32/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.33/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.33/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.34/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.34/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.35/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.35/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.36/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.36/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.37/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.37/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.38/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.38/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.39/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.39/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.40/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.40/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.41/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.41/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.42/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.42/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.43/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.43/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.44/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.44/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.45/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.45/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.46/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.46/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.47/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.47/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.48/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.48/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.49/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.49/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.50/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.50/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.51/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.51/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.52/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.52/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.53/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.53/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.54/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.54/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.55/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.55/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.56/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.56/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.57/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.57/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.58/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.58/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.59/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.59/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.60/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.60/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.61/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.61/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.62/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.62/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.63/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.63/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.64/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.64/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.65/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.65/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.66/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.66/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.67/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.67/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.68/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.68/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.69/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.69/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.70/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.70/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.71/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.71/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.72/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.72/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.73/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.73/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.74/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.74/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.75/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.75/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.76/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.76/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.77/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.77/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.78/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.78/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.79/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.79/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.80/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.80/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.81/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.81/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.82/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.82/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.83/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.83/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.84/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.84/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.85/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.85/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.86/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.86/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.87/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.87/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.88/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.88/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.89/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.89/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.90/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.90/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.91/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.91/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.92/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.92/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.93/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.93/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.94/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.94/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.95/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.95/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.96/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.96/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.97/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.97/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.98/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.98/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.99/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.99/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.100/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.100/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.101/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.101/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.102/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.102/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.103/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.103/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.104/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.104/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.105/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.105/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.106/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.106/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.107/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.107/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.108/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.108/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.109/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.109/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.110/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.110/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.111/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.111/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.112/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.112/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.113/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.113/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.114/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.114/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.115/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.115/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.116/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.116/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.117/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.117/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.118/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.118/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.119/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.119/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.120/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.120/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.121/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.121/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.122/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.122/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.123/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.123/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.124/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.124/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.125/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.125/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.126/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.126/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.127/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.127/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.128/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.128/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.129/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.129/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.130/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.130/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.131/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.131/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.132/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.132/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.133/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.133/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.134/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.134/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.135/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.135/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.136/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.136/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.137/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.137/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.138/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.138/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.139/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.139/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.140/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.140/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.141/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.141/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.142/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.142/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.143/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.143/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.144/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.144/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.145/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.145/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.146/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.146/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.147/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.147/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.148/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.148/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.149/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.149/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.150/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.150/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.151/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.151/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.152/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.152/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.153/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.153/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.154/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.154/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.155/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.155/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.156/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.156/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.157/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.157/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.158/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.158/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.159/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.159/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.160/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.160/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.161/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.161/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.162/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.162/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.163/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.163/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.164/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.164/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.165/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.165/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.166/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.166/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.167/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.167/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.168/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.168/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.169/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.169/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.170/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.170/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.171/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.171/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.172/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.172/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.173/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.173/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.174/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.174/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.175/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.175/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.176/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.176/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.177/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.177/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.178/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.178/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.179/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.179/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.180/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.180/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.181/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.181/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.182/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.182/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.183/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.183/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.184/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.184/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.185/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.185/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.186/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.186/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.187/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.187/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.188/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.188/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.189/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.189/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.190/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.190/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.191/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.191/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.192/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.192/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.193/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.193/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.194/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.194/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.195/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.195/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.196/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.196/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.197/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.197/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.198/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.198/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.199/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.199/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.200/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.200/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.111.0/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.111.0/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:31 	Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.4.0/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.4.0/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:06 	Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.100.0/25 (2 entries, 1 announced)
        TSI:
        KRT in-kernel 192.168.100.0/25 -> {10.169.14.121}
                *OSPF   Preference: 150/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int Ext>
                        Local AS: 65171 
                        Age: 2w5d 16:18:45 	Metric: 32000 
                        Validation State: unverified 
                            Tag: 65000500 
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.100.252/32 (2 entries, 1 announced)
        TSI:
        KRT in-kernel 192.168.100.252/32 -> {10.169.14.121}
                *OSPF   Preference: 150/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int Ext>
                        Local AS: 65171 
                        Age: 2w5d 16:18:45 	Metric: 32000 
                        Validation State: unverified 
                            Tag: 65000500 
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.36.48/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.36.48/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 10100 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.36.56/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.36.56/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 10100 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.36.119/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.36.119/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 10101 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.36.120/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.36.120/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 10101 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        224.0.0.5/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 224.0.0.5/32 -> {}
                *OSPF   Preference: 10
                        Next hop type: MultiRecv, Next hop index: 0
                        Address: 0xbb66cd4
                        Next-hop reference count: 9
                        State: <Active NoReadvrt Int>
                        Local AS: 65171 
                        Age: 29w5d 23:06:46 	Metric: 1 
                        Validation State: unverified 
                        Task: OSPF I/O./var/run/ppmd_control
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        inet.3: 11 destinations, 11 routes (11 active, 0 holddown, 0 hidden)

        10.100.5.5/32 (1 entry, 1 announced)
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc14594
                        Next-hop reference count: 1
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Push 17000
                        Label TTL action: no-prop-ttl
                        Load balance label: Label 17000: None; 
                        Label element ptr: 0xc5f6ec0
                        Label parent element ptr: 0x0
                        Label element references: 2
                        Label element child references: 1
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Push 17000, Push 1650, Push 1913(top)
                        Label TTL action: no-prop-ttl, no-prop-ttl, no-prop-ttl(top)
                        Load balance label: Label 17000: None; Label 1650: None; Label 1913: None; 
                        Label element ptr: 0xc5c9a00
                        Label parent element ptr: 0x0
                        Label element references: 2
                        Label element child references: 1
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 5:30:11 	Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 3-Resolve tree 1 4-Resolve tree 3 
                        AS path: I 

        10.19.198.239/32 (1 entry, 1 announced)
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc14714
                        Next-hop reference count: 1
                        Next hop: 10.19.198.26 via ge-0/0/2.0 weight 0x1, selected
                        Label element ptr: 0xc5cda38
                        Label parent element ptr: 0x0
                        Label element references: 14
                        Label element child references: 10
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Push 16073
                        Label TTL action: no-prop-ttl
                        Load balance label: Label 16073: None; 
                        Label element ptr: 0xc5ee888
                        Label parent element ptr: 0x0
                        Label element references: 2
                        Label element child references: 1
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 1w5d 22:11:26 	Metric: 1001 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 3-Resolve tree 1 4-Resolve tree 3 
                        AS path: I 

        10.34.2.250/32 (1 entry, 1 announced)
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xdfadab4
                        Next-hop reference count: 1
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Push 16061
                        Label TTL action: no-prop-ttl
                        Load balance label: Label 16061: None; 
                        Label element ptr: 0xc5f6e20
                        Label parent element ptr: 0x0
                        Label element references: 2
                        Label element child references: 1
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:26 	Metric: 200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 3-Resolve tree 1 4-Resolve tree 3 
                        AS path: I 

        10.34.2.251/32 (1 entry, 1 announced)
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbb6ba14
                        Next-hop reference count: 1
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Push 16062
                        Label TTL action: no-prop-ttl
                        Load balance label: Label 16062: None; 
                        Label element ptr: 0xc5f6df8
                        Label parent element ptr: 0x0
                        Label element references: 2
                        Label element child references: 1
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:26 	Metric: 205 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 3-Resolve tree 1 4-Resolve tree 3 
                        AS path: I 

        10.169.196.241/32 (1 entry, 1 announced)
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc14c14
                        Next-hop reference count: 1
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Push 16063
                        Label TTL action: no-prop-ttl
                        Load balance label: Label 16063: None; 
                        Label element ptr: 0xc5c8128
                        Label parent element ptr: 0x0
                        Label element references: 2
                        Label element child references: 1
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Push 16063, Push 1650, Push 1913(top)
                        Label TTL action: no-prop-ttl, no-prop-ttl, no-prop-ttl(top)
                        Load balance label: Label 16063: None; Label 1650: None; Label 1913: None; 
                        Label element ptr: 0xc5f70f0
                        Label parent element ptr: 0x0
                        Label element references: 2
                        Label element child references: 1
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:26 	Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 3-Resolve tree 1 4-Resolve tree 3 
                        AS path: I 

        10.169.14.240/32 (1 entry, 1 announced)
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc1bd14
                        Next-hop reference count: 2
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label element ptr: 0xc5cda38
                        Label parent element ptr: 0x0
                        Label element references: 14
                        Label element child references: 10
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Push 16051, Push 1913(top)
                        Label TTL action: no-prop-ttl, no-prop-ttl(top)
                        Load balance label: Label 16051: None; Label 1913: None; 
                        Label element ptr: 0xc5ee6f8
                        Label parent element ptr: 0x0
                        Label element references: 6
                        Label element child references: 5
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:51 	Metric: 100 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 3-Resolve tree 1 4-Resolve tree 3 6-Resolve_IGP_FRR task 
                        AS path: I 

        10.169.14.241/32 (1 entry, 1 announced)
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xdfb0bd4
                        Next-hop reference count: 1
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Push 16052
                        Label TTL action: no-prop-ttl
                        Load balance label: Label 16052: None; 
                        Label element ptr: 0xc5f6d80
                        Label parent element ptr: 0x0
                        Label element references: 2
                        Label element child references: 1
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 105 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 3-Resolve tree 1 4-Resolve tree 3 
                        AS path: I 

        10.189.5.253/32 (1 entry, 1 announced)
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xdfb2af4
                        Next-hop reference count: 4
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0x1, selected
                        Label element ptr: 0xc5cda38
                        Label parent element ptr: 0x0
                        Label element references: 14
                        Label element child references: 10
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:56 	Metric: 5 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 3-Resolve tree 1 4-Resolve tree 3 6-Resolve_IGP_FRR task 
                        AS path: I 

        mpls.0: 44 destinations, 44 routes (44 active, 0 holddown, 0 hidden)

        2567 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 2567   /52 -> {list:Pop      , Swap 16051, Push 1913(top)}
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc32314
                        Next-hop reference count: 2
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8a8
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Swap 16051, Push 1913(top)
                        Label TTL action: no-prop-ttl, no-prop-ttl(top)
                        Load balance label: Label 16051: None; Label 1913: None; 
                        Label element ptr: 0xc5f4b98
                        Label parent element ptr: 0x0
                        Label element references: 3
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 1w6d 20:26:02 	Metric: 0 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        2567(S=0) (1 entry, 1 announced)
        TSI:
        KRT in-kernel 2567   /56 -> {list:Pop      , Swap 16051, Push 1913(top)}
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc1d894
                        Next-hop reference count: 2
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8d0
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Swap 16051, Push 1913(top)
                        Label TTL action: no-prop-ttl, no-prop-ttl(top)
                        Load balance label: Label 16051: None; Label 1913: None; 
                        Label element ptr: 0xc5f4b98
                        Label parent element ptr: 0x0
                        Label element references: 3
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 1w6d 20:26:02 	Metric: 0 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        2568 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 2568   /52 -> {Pop      }
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 603
                        Address: 0xdfaa1b4
                        Next-hop reference count: 2
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8a8
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 0 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        2568(S=0) (1 entry, 1 announced)
        TSI:
        KRT in-kernel 2568   /56 -> {Pop      }
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 618
                        Address: 0xdfadc34
                        Next-hop reference count: 2
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8d0
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 0 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        16051 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 16051  /52 -> {list:Pop      , Swap 16051, Push 1913(top)}
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc32314
                        Next-hop reference count: 2
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8a8
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Swap 16051, Push 1913(top)
                        Label TTL action: no-prop-ttl, no-prop-ttl(top)
                        Load balance label: Label 16051: None; Label 1913: None; 
                        Label element ptr: 0xc5f4b98
                        Label parent element ptr: 0x0
                        Label element references: 3
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:51 	Metric: 100 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        16051(S=0) (1 entry, 1 announced)
        TSI:
        KRT in-kernel 16051  /56 -> {list:Pop      , Swap 16051, Push 1913(top)}
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc1d894
                        Next-hop reference count: 2
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8d0
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Swap 16051, Push 1913(top)
                        Label TTL action: no-prop-ttl, no-prop-ttl(top)
                        Load balance label: Label 16051: None; Label 1913: None; 
                        Label element ptr: 0xc5f4b98
                        Label parent element ptr: 0x0
                        Label element references: 3
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:51 	Metric: 100 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        16052 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 16052  /52 -> {Swap 16052}
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 626
                        Address: 0xdbdb334
                        Next-hop reference count: 2
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Swap 16052
                        Load balance label: Label 16052: None; 
                        Label element ptr: 0xc5f6d08
                        Label parent element ptr: 0x0
                        Label element references: 1
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:43:35 	Metric: 105 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        16061 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 16061  /52 -> {Swap 16061}
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 616
                        Address: 0xdfadbd4
                        Next-hop reference count: 2
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Swap 16061
                        Load balance label: Label 16061: None; 
                        Label element ptr: 0xc5f6cb8
                        Label parent element ptr: 0x0
                        Label element references: 1
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:26 	Metric: 200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        16062 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 16062  /52 -> {Swap 16062}
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 619
                        Address: 0xbb6e474
                        Next-hop reference count: 2
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Swap 16062
                        Load balance label: Label 16062: None; 
                        Label element ptr: 0xc5f6c90
                        Label parent element ptr: 0x0
                        Label element references: 1
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:26 	Metric: 205 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        16063 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 16063  /52 -> {list:Swap 16063, Swap 16063, Push 1650, Push 1913(top)}
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc14894
                        Next-hop reference count: 1
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Swap 16063
                        Load balance label: Label 16063: None; 
                        Label element ptr: 0xc5c8150
                        Label parent element ptr: 0x0
                        Label element references: 2
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Swap 16063, Push 1650, Push 1913(top)
                        Label TTL action: no-prop-ttl, no-prop-ttl, no-prop-ttl(top)
                        Load balance label: Label 16063: None; Label 1650: None; Label 1913: None; 
                        Label element ptr: 0xc5f72d0
                        Label parent element ptr: 0x0
                        Label element references: 2
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:15:26 	Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        16072 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 16072  /52 -> {Pop      }
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 622
                        Address: 0xe336114
                        Next-hop reference count: 6
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8a8
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x140
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:56 	Metric: 5 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        16072(S=0) (1 entry, 1 announced)
        TSI:
        KRT in-kernel 16072  /56 -> {Pop      }
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 624
                        Address: 0xe34b8f4
                        Next-hop reference count: 6
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8d0
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x140
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:56 	Metric: 5 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        16073 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 16073  /52 -> {list:Pop      , Swap 16073}
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc16b94
                        Next-hop reference count: 2
                        Next hop: 10.19.198.26 via ge-0/0/2.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8a8
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Swap 16073
                        Load balance label: Label 16073: None; 
                        Label element ptr: 0xc5ee928
                        Label parent element ptr: 0x0
                        Label element references: 3
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 1w5d 22:11:26 	Metric: 1001 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        16073(S=0) (1 entry, 1 announced)
        TSI:
        KRT in-kernel 16073  /56 -> {list:Pop      , Swap 16073}
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc16c14
                        Next-hop reference count: 2
                        Next hop: 10.19.198.26 via ge-0/0/2.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8d0
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Swap 16073
                        Load balance label: Label 16073: None; 
                        Label element ptr: 0xc5ee928
                        Label parent element ptr: 0x0
                        Label element references: 3
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 1w5d 22:11:26 	Metric: 1001 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        17000 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 17000  /52 -> {list:Swap 17000, Swap 17000, Push 1650, Push 1913(top)}
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc14b94
                        Next-hop reference count: 1
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Label operation: Swap 17000
                        Load balance label: Label 17000: None; 
                        Label element ptr: 0xc5f6d30
                        Label parent element ptr: 0x0
                        Label element references: 2
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Swap 17000, Push 1650, Push 1913(top)
                        Label TTL action: no-prop-ttl, no-prop-ttl, no-prop-ttl(top)
                        Load balance label: Label 17000: None; Label 1650: None; Label 1913: None; 
                        Label element ptr: 0xc5f34a0
                        Label parent element ptr: 0x0
                        Label element references: 2
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 5:30:11 	Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        28985 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 28985  /52 -> {Pop      }
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 622
                        Address: 0xe336114
                        Next-hop reference count: 6
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8a8
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x140
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:56 	Metric: 0 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        28985(S=0) (1 entry, 1 announced)
        TSI:
        KRT in-kernel 28985  /56 -> {Pop      }
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 624
                        Address: 0xe34b8f4
                        Next-hop reference count: 6
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8d0
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x140
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:56 	Metric: 0 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        28986 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 28986  /52 -> {Pop      }
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 622
                        Address: 0xe336114
                        Next-hop reference count: 6
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8a8
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x140
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:56 	Metric: 0 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        28986(S=0) (1 entry, 1 announced)
        TSI:
        KRT in-kernel 28986  /56 -> {Pop      }
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 624
                        Address: 0xe34b8f4
                        Next-hop reference count: 6
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8d0
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x140
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:56 	Metric: 0 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        167966 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 167966 /52 -> {list:Pop      , Swap 16073}
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc16b94
                        Next-hop reference count: 2
                        Next hop: 10.19.198.26 via ge-0/0/2.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8a8
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Swap 16073
                        Load balance label: Label 16073: None; 
                        Label element ptr: 0xc5ee928
                        Label parent element ptr: 0x0
                        Label element references: 3
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 1w5d 22:11:26 	Metric: 0 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        167966(S=0) (1 entry, 1 announced)
        TSI:
        KRT in-kernel 167966 /56 -> {list:Pop      , Swap 16073}
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 0
                        Address: 0xbc16c14
                        Next-hop reference count: 2
                        Next hop: 10.19.198.26 via ge-0/0/2.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8d0
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0xf000
                        Label operation: Swap 16073
                        Load balance label: Label 16073: None; 
                        Label element ptr: 0xc5ee928
                        Label parent element ptr: 0x0
                        Label element references: 3
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x0
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 1w5d 22:11:26 	Metric: 0 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        167967 (1 entry, 1 announced)
        TSI:
        KRT in-kernel 167967 /52 -> {Pop      }
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 655
                        Address: 0xdbd53f4
                        Next-hop reference count: 2
                        Next hop: 10.19.198.26 via ge-0/0/2.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8a8
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x1ae
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 1w5d 22:11:35 	Metric: 0 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        167967(S=0) (1 entry, 1 announced)
        TSI:
        KRT in-kernel 167967 /56 -> {Pop      }
                *L-OSPF Preference: 10/5
                        Next hop type: Router, Next hop index: 656
                        Address: 0xdbd5334
                        Next-hop reference count: 2
                        Next hop: 10.19.198.26 via ge-0/0/2.0 weight 0x1, selected
                        Label operation: Pop      
                        Load balance label: None; 
                        Label element ptr: 0xc5cd8d0
                        Label parent element ptr: 0x0
                        Label element references: 7
                        Label element child references: 0
                        Label element lsp id: 0
                        Session Id: 0x1ae
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 1w5d 22:11:35 	Metric: 0 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (1): 1-KRT 
                        AS path: I 

        inet6.0: 22 destinations, 23 routes (22 active, 0 holddown, 0 hidden)

        ::/0 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel ::/0 -> {fe80::250:56ff:fe8d:72bd}
                *OSPF3  Preference: 150
                        Next hop type: Router, Next hop index: 628
                        Address: 0xe34bb94
                        Next-hop reference count: 19
                        Next hop: fe80::250:56ff:fe8d:72bd via ge-0/0/1.0, selected
                        Session Id: 0x144
                        State: <Active Int Ext>
                        Local AS: 65171 
                        Age: 3w0d 18:21:55 	Metric: 101 
                        Validation State: unverified 
                            Tag: 0 
                        Task: OSPF3
                        Announcement bits (3): 0-KRT 3-LDP 5-Resolve tree 5 
                        AS path: I 

        2001:db8:6aa8:6a53::1001/128 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 2001:db8:6aa8:6a53::1001/128 -> {fe80::250:56ff:fe8d:72bd}
                *OSPF3  Preference: 150
                        Next hop type: Router, Next hop index: 628
                        Address: 0xe34bb94
                        Next-hop reference count: 19
                        Next hop: fe80::250:56ff:fe8d:72bd via ge-0/0/1.0, selected
                        Session Id: 0x144
                        State: <Active Int Ext>
                        Local AS: 65171 
                        Age: 3w0d 18:21:55 	Metric: 150 
                        Validation State: unverified 
                            Tag: 0 
                        Task: OSPF3
                        Announcement bits (3): 0-KRT 3-LDP 5-Resolve tree 5 
                        AS path: I 

        2001:db8:b0f8:ca45::13/128 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 2001:db8:b0f8:ca45::13/128 -> {fe80::250:56ff:fe8d:72bd}
                *OSPF3  Preference: 10
                        Next hop type: Router, Next hop index: 628
                        Address: 0xe34bb94
                        Next-hop reference count: 19
                        Next hop: fe80::250:56ff:fe8d:72bd via ge-0/0/1.0, selected
                        Session Id: 0x144
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:17 	Metric: 200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF3
                        Announcement bits (3): 0-KRT 3-LDP 5-Resolve tree 5 
                        AS path: I 

        2001:db8:b0f8:ca45::14/128 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 2001:db8:b0f8:ca45::14/128 -> {fe80::250:56ff:fe8d:72bd}
                *OSPF3  Preference: 10
                        Next hop type: Router, Next hop index: 628
                        Address: 0xe34bb94
                        Next-hop reference count: 19
                        Next hop: fe80::250:56ff:fe8d:72bd via ge-0/0/1.0, selected
                        Session Id: 0x144
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:17 	Metric: 205 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF3
                        Announcement bits (3): 0-KRT 3-LDP 5-Resolve tree 5 
                        AS path: I 

        2001:db8:b0f8:3ab::/64 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 2001:db8:b0f8:3ab::/64 -> {fe80::250:56ff:fe8d:72bd}
                *OSPF3  Preference: 10
                        Next hop type: Router, Next hop index: 628
                        Address: 0xe34bb94
                        Next-hop reference count: 19
                        Next hop: fe80::250:56ff:fe8d:72bd via ge-0/0/1.0, selected
                        Session Id: 0x144
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:17 	Metric: 205 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF3
                        Announcement bits (3): 0-KRT 3-LDP 5-Resolve tree 5 
                        AS path: I 

        2001:db8:eb18:ca45::1/128 (2 entries, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 2001:db8:eb18:ca45::1/128 -> {2001:db8:eb18:6337::1}
        OSPF3 realm ipv6-unicast area : 0.0.0.0, LSA ID : 0.0.0.1, LSA type : Extern
                OSPF3  Preference: 10
                        Next hop type: Router, Next hop index: 628
                        Address: 0xe34bb94
                        Next-hop reference count: 19
                        Next hop: fe80::250:56ff:fe8d:72bd via ge-0/0/1.0, selected
                        Session Id: 0x144
                        State: <Int>
                        Inactive reason: Route Preference
                        Local AS: 65171 
                        Age: 3w0d 18:21:55 	Metric: 100 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF3
                        AS path: I 

        2001:db8:eb18:ca45::2/128 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 2001:db8:eb18:ca45::2/128 -> {fe80::250:56ff:fe8d:72bd}
                *OSPF3  Preference: 10
                        Next hop type: Router, Next hop index: 628
                        Address: 0xe34bb94
                        Next-hop reference count: 19
                        Next hop: fe80::250:56ff:fe8d:72bd via ge-0/0/1.0, selected
                        Session Id: 0x144
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:55 	Metric: 105 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF3
                        Announcement bits (3): 0-KRT 3-LDP 5-Resolve tree 5 
                        AS path: I 

        2001:db8:eb18:e26e::/64 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 2001:db8:eb18:e26e::/64 -> {fe80::250:56ff:fe8d:72bd}
                *OSPF3  Preference: 10
                        Next hop type: Router, Next hop index: 628
                        Address: 0xe34bb94
                        Next-hop reference count: 19
                        Next hop: fe80::250:56ff:fe8d:72bd via ge-0/0/1.0, selected
                        Session Id: 0x144
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:21:55 	Metric: 105 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF3
                        Announcement bits (3): 0-KRT 3-LDP 5-Resolve tree 5 
                        AS path: I 

        2001:db8:eb18:f5e6::/64 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 2001:db8:eb18:f5e6::/64 -> {fe80::250:56ff:fe8d:72bd}
                *OSPF3  Preference: 10
                        Next hop type: Router, Next hop index: 628
                        Address: 0xe34bb94
                        Next-hop reference count: 19
                        Next hop: fe80::250:56ff:fe8d:72bd via ge-0/0/1.0, selected
                        Session Id: 0x144
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 5:30:12 	Metric: 225 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF3
                        Announcement bits (3): 0-KRT 3-LDP 5-Resolve tree 5 
                        AS path: I 

        2001:db8:eb18:6d57::/64 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 2001:db8:eb18:6d57::/64 -> {fe80::250:56ff:fe8d:53c0}
                *OSPF3  Preference: 10
                        Next hop type: Router, Next hop index: 621
                        Address: 0xdfa4454
                        Next-hop reference count: 4
                        Next hop: fe80::250:56ff:fe8d:53c0 via ge-0/0/0.0, selected
                        Session Id: 0x142
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:22:00 	Metric: 125 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF3
                        Announcement bits (3): 0-KRT 3-LDP 5-Resolve tree 5 
                        AS path: I 

        2001:db8:eb18:9627::/64 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 2001:db8:eb18:9627::/64 -> {fe80::250:56ff:fe8d:72bd}
                *OSPF3  Preference: 10
                        Next hop type: Router, Next hop index: 628
                        Address: 0xe34bb94
                        Next-hop reference count: 19
                        Next hop: fe80::250:56ff:fe8d:72bd via ge-0/0/1.0, selected
                        Session Id: 0x144
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:10:17 	Metric: 200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF3
                        Announcement bits (3): 0-KRT 3-LDP 5-Resolve tree 5 
                        AS path: I 

        2001:db8:223c:ca45::c/128 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 2001:db8:223c:ca45::c/128 -> {fe80::250:56ff:fe8d:53c0}
                *OSPF3  Preference: 10
                        Next hop type: Router, Next hop index: 621
                        Address: 0xdfa4454
                        Next-hop reference count: 4
                        Next hop: fe80::250:56ff:fe8d:53c0 via ge-0/0/0.0, selected
                        Session Id: 0x142
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:22:00 	Metric: 5 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF3
                        Announcement bits (3): 0-KRT 3-LDP 5-Resolve tree 5 
                        AS path: I 

        ff02::5/128 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel ff02::5/128 -> {}
                *OSPF3  Preference: 10
                        Next hop type: MultiRecv, Next hop index: 0
                        Address: 0xbb66cd4
                        Next-hop reference count: 9
                        State: <Active NoReadvrt Int>
                        Local AS: 65171 
                        Age: 29w5d 23:06:46 	Metric: 1 
                        Validation State: unverified 
                        Task: OSPF3 I/O./var/run/ppmd_control
                        Announcement bits (3): 0-KRT 3-LDP 5-Resolve tree 5 
                        AS path: I 
    '''}

    golden_parsed_output = {
        "route-information": {
            "route-table": [
                {
                    "active-route-count": "929",
                    "destination-count": "929",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "0.0.0.0/0",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "101",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int Ext",
                                "rt-tag": "0",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 0.0.0.0/0 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.1.0.0/24",
                            "rt-entry": {
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "inactive-reason": "Route Preference",
                                "local-as": "65171",
                                "metric": "20",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Int Ext",
                                "rt-tag": "0",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "2",
                                "@junos:format": "2 entries"
                            },
                            "rt-state": "FlashAll"
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.36.3.3/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.36.3.3/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.16.0.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:26"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1200",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.16.0.0/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.100.5.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:26"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.100.5.5/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.19.198.28/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:56"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1005",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "140",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdbe48d4",
                                "nh-index": "614",
                                "nh-reference-count": "6",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.19.198.28/30 -> {10.189.5.94}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.19.198.239/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w5d 22:11:35"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1001",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "1ae",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdbc0d54",
                                "nh-index": "605",
                                "nh-reference-count": "4",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.19.198.239/32 -> {10.19.198.26}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.174.132.237/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "150",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int Ext",
                                "rt-tag": "0",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.174.132.237/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.34.2.200/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:26"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "205",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.34.2.200/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.34.2.250/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:26"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "200",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.34.2.250/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.34.2.251/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:26"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "205",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.34.2.251/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.15.0.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w5d 22:11:35"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1001",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "1ae",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdbc0d54",
                                "nh-index": "605",
                                "nh-reference-count": "4",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.15.0.0/30 -> {10.19.198.26}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.64.0.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.64.0.0/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.196.212/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:26"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1200",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.196.212/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.196.216/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:26"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1205",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.196.216/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.196.241/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.196.241/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.16/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "105",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.16/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.32/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 5:30:23"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "225",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.32/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.128/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:56"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "125",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "140",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdbe48d4",
                                "nh-index": "614",
                                "nh-reference-count": "6",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.128/30 -> {10.189.5.94}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.156/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:28"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "200",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.156/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.240/32",
                            "rt-entry": {
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "inactive-reason": "Route Preference",
                                "local-as": "65171",
                                "metric": "100",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "2",
                                "@junos:format": "2 entries"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.240/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.241/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "105",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.241/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.242/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "100",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.242/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.243/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "105",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.243/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.189.5.253/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:56"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "5",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "140",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdbe48d4",
                                "nh-index": "614",
                                "nh-reference-count": "6",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.189.5.253/32 -> {10.189.5.94}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:26"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1200",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.0/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.0/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.0/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.1/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.1/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.2/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.2/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.3/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.3/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.4/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.4/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.5/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.6/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.6/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.7/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.7/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.8/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.8/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.9/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.9/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.10/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.10/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.11/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.11/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.12/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.12/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.13/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.13/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.14/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.14/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.15/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.15/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.16/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.16/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.17/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.17/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.18/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.18/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.19/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.19/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.20/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.20/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.21/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.21/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.22/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.22/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.23/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.23/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.24/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.24/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.25/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.25/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.26/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.26/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.27/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.27/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.28/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.28/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.29/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.29/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.30/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.30/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.31/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.31/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.32/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.32/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.33/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.33/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.34/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.34/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.35/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.35/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.36/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.36/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.37/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.37/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.38/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.38/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.39/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.39/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.40/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.40/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.41/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.41/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.42/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.42/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.43/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.43/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.44/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.44/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.45/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.45/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.46/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.46/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.47/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.47/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.48/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.48/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.49/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.49/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.50/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.50/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.51/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.51/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.52/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.52/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.53/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.53/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.54/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.54/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.55/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.55/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.56/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.56/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.57/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.57/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.58/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.58/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.59/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.59/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.60/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.60/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.61/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.61/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.62/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.62/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.63/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.63/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.64/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.64/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.65/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.65/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.66/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.66/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.67/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.67/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.68/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.68/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.69/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.69/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.70/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.70/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.71/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.71/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.72/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.72/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.73/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.73/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.74/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.74/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.75/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.75/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.76/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.76/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.77/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.77/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.78/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.78/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.79/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.79/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.80/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.80/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.81/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.81/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.82/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.82/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.83/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.83/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.84/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.84/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.85/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.85/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.86/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.86/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.87/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.87/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.88/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.88/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.89/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.89/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.90/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.90/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.91/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.91/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.92/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.92/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.93/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.93/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.94/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.94/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.95/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.95/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.96/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.96/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.97/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.97/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.98/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.98/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.99/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.99/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.100/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.100/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.101/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.101/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.102/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.102/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.103/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.103/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.104/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.104/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.105/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.105/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.106/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.106/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.107/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.107/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.108/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.108/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.109/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.109/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.110/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.110/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.111/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.111/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.112/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.112/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.113/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.113/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.114/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.114/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.115/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.115/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.116/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.116/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.117/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.117/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.118/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.118/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.119/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.119/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.120/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.120/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.121/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.121/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.122/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.122/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.123/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.123/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.124/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.124/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.125/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.125/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.126/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.126/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.127/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.127/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.128/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.128/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.129/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.129/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.130/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.130/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.131/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.131/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.132/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.132/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.133/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.133/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.134/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.134/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.135/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.135/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.136/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.136/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.137/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.137/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.138/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.138/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.139/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.139/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.140/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.140/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.141/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.141/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.142/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.142/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.143/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.143/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.144/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.144/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.145/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.145/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.146/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.146/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.147/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.147/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.148/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.148/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.149/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.149/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.150/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.150/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.151/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.151/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.152/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.152/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.153/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.153/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.154/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.154/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.155/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.155/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.156/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.156/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.157/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.157/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.158/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.158/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.159/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.159/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.160/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.160/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.161/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.161/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.162/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.162/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.163/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.163/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.164/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.164/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.165/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.165/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.166/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.166/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.167/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.167/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.168/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.168/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.169/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.169/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.170/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.170/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.171/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.171/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.172/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.172/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.173/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.173/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.174/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.174/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.175/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.175/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.176/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.176/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.177/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.177/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.178/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.178/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.179/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.179/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.180/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.180/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.181/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.181/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.182/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.182/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.183/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.183/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.184/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.184/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.185/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.185/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.186/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.186/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.187/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.187/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.188/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.188/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.189/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.189/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.190/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.190/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.191/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.191/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.192/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.192/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.193/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.193/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.194/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.194/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.195/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.195/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.196/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.196/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.197/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.197/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.198/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.198/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.199/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.199/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.200/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.200/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.111.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:31"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.111.0/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.4.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:06"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.4.0/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.100.0/25",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w5d 16:18:45"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "32000",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int Ext",
                                "rt-tag": "65000500",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "2",
                                "@junos:format": "2 entries"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.100.0/25 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.100.252/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w5d 16:18:45"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "32000",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int Ext",
                                "rt-tag": "65000500",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "2",
                                "@junos:format": "2 entries"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.100.252/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.36.48/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "10100",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.36.48/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.36.56/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "10100",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.36.56/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.36.119/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "10101",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.36.119/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.36.120/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "10101",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.36.120/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "224.0.0.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "29w5d 23:06:46"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1",
                                "nh-address": "0xbb66cd4",
                                "nh-index": "0",
                                "nh-reference-count": "9",
                                "nh-type": "MultiRecv",
                                "preference": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active NoReadvrt Int",
                                "task-name": "OSPF I/O./var/run/ppmd_control",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 224.0.0.5/32 -> {}"
                            }
                        }
                    ],
                    "table-name": "inet.0",
                    "total-route-count": "1615"
                },
                {
                    "active-route-count": "11",
                    "destination-count": "11",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.100.5.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 5:30:11"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "3-Resolve tree 1 4-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "label-element": "0xc5f6ec0",
                                        "label-element-childcount": "1",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "2",
                                        "label-ttl-action": "no-prop-ttl",
                                        "load-balance-label": "Label 17000: None;",
                                        "mpls-label": "Push 17000",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5c9a00",
                                        "label-element-childcount": "1",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "2",
                                        "label-ttl-action": "no-prop-ttl, no-prop-ttl, no-prop-ttl(top)",
                                        "load-balance-label": "Label 17000: None; Label 1650: None; Label 1913: None;",
                                        "mpls-label": "Push 17000, Push 1650, Push 1913(top)",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc14594",
                                "nh-index": "0",
                                "nh-reference-count": "1",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.19.198.239/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w5d 22:11:26"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "3-Resolve tree 1 4-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1001",
                                "nh": [
                                    {
                                        "label-element": "0xc5cda38",
                                        "label-element-childcount": "10",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "14",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5ee888",
                                        "label-element-childcount": "1",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "2",
                                        "label-ttl-action": "no-prop-ttl",
                                        "load-balance-label": "Label 16073: None;",
                                        "mpls-label": "Push 16073",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc14714",
                                "nh-index": "0",
                                "nh-reference-count": "1",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.34.2.250/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:26"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "3-Resolve tree 1 4-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "200",
                                "nh": [
                                    {
                                        "label-element": "0xc5f6e20",
                                        "label-element-childcount": "1",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "2",
                                        "label-ttl-action": "no-prop-ttl",
                                        "load-balance-label": "Label 16061: None;",
                                        "mpls-label": "Push 16061",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfadab4",
                                "nh-index": "0",
                                "nh-reference-count": "1",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.34.2.251/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:26"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "3-Resolve tree 1 4-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "205",
                                "nh": [
                                    {
                                        "label-element": "0xc5f6df8",
                                        "label-element-childcount": "1",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "2",
                                        "label-ttl-action": "no-prop-ttl",
                                        "load-balance-label": "Label 16062: None;",
                                        "mpls-label": "Push 16062",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xbb6ba14",
                                "nh-index": "0",
                                "nh-reference-count": "1",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.196.241/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:26"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "3-Resolve tree 1 4-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "label-element": "0xc5c8128",
                                        "label-element-childcount": "1",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "2",
                                        "label-ttl-action": "no-prop-ttl",
                                        "load-balance-label": "Label 16063: None;",
                                        "mpls-label": "Push 16063",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5f70f0",
                                        "label-element-childcount": "1",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "2",
                                        "label-ttl-action": "no-prop-ttl, no-prop-ttl, no-prop-ttl(top)",
                                        "load-balance-label": "Label 16063: None; Label 1650: None; Label 1913: None;",
                                        "mpls-label": "Push 16063, Push 1650, Push 1913(top)",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc14c14",
                                "nh-index": "0",
                                "nh-reference-count": "1",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.240/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:51"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "3-Resolve tree 1 4-Resolve tree 3 6-Resolve_IGP_FRR task",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "100",
                                "nh": [
                                    {
                                        "label-element": "0xc5cda38",
                                        "label-element-childcount": "10",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "14",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5ee6f8",
                                        "label-element-childcount": "5",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "6",
                                        "label-ttl-action": "no-prop-ttl, no-prop-ttl(top)",
                                        "load-balance-label": "Label 16051: None; Label 1913: None;",
                                        "mpls-label": "Push 16051, Push 1913(top)",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc1bd14",
                                "nh-index": "0",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.241/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "3-Resolve tree 1 4-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "105",
                                "nh": [
                                    {
                                        "label-element": "0xc5f6d80",
                                        "label-element-childcount": "1",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "2",
                                        "label-ttl-action": "no-prop-ttl",
                                        "load-balance-label": "Label 16052: None;",
                                        "mpls-label": "Push 16052",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfb0bd4",
                                "nh-index": "0",
                                "nh-reference-count": "1",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.189.5.253/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:56"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "3-Resolve tree 1 4-Resolve tree 3 6-Resolve_IGP_FRR task",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "5",
                                "nh": [
                                    {
                                        "label-element": "0xc5cda38",
                                        "label-element-childcount": "10",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "14",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfb2af4",
                                "nh-index": "0",
                                "nh-reference-count": "4",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            }
                        }
                    ],
                    "table-name": "inet.3",
                    "total-route-count": "11"
                },
                {
                    "active-route-count": "44",
                    "destination-count": "44",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2567",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w6d 20:26:02"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "0",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8a8",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5f4b98",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "3",
                                        "label-ttl-action": "no-prop-ttl, no-prop-ttl(top)",
                                        "load-balance-label": "Label 16051: None; Label 1913: None;",
                                        "mpls-label": "Swap 16051, Push 1913(top)",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc32314",
                                "nh-index": "0",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 2567   /52 -> {list:Pop      , Swap 16051, Push 1913(top)}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2567(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w6d 20:26:02"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "0",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8d0",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5f4b98",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "3",
                                        "label-ttl-action": "no-prop-ttl, no-prop-ttl(top)",
                                        "load-balance-label": "Label 16051: None; Label 1913: None;",
                                        "mpls-label": "Swap 16051, Push 1913(top)",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc1d894",
                                "nh-index": "0",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 2567   /56 -> {list:Pop      , Swap 16051, Push 1913(top)}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2568",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "0",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8a8",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfaa1b4",
                                "nh-index": "603",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 2568   /52 -> {Pop      }"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2568(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "0",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8d0",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfadc34",
                                "nh-index": "618",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 2568   /56 -> {Pop      }"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "16051",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:51"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "100",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8a8",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5f4b98",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "3",
                                        "label-ttl-action": "no-prop-ttl, no-prop-ttl(top)",
                                        "load-balance-label": "Label 16051: None; Label 1913: None;",
                                        "mpls-label": "Swap 16051, Push 1913(top)",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc32314",
                                "nh-index": "0",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 16051  /52 -> {list:Pop      , Swap 16051, Push 1913(top)}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "16051(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:51"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "100",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8d0",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5f4b98",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "3",
                                        "label-ttl-action": "no-prop-ttl, no-prop-ttl(top)",
                                        "load-balance-label": "Label 16051: None; Label 1913: None;",
                                        "mpls-label": "Swap 16051, Push 1913(top)",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc1d894",
                                "nh-index": "0",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 16051  /56 -> {list:Pop      , Swap 16051, Push 1913(top)}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "16052",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:43:35"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "105",
                                "nh": [
                                    {
                                        "label-element": "0xc5f6d08",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "1",
                                        "load-balance-label": "Label 16052: None;",
                                        "mpls-label": "Swap 16052",
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdbdb334",
                                "nh-index": "626",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 16052  /52 -> {Swap 16052}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "16061",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:26"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "200",
                                "nh": [
                                    {
                                        "label-element": "0xc5f6cb8",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "1",
                                        "load-balance-label": "Label 16061: None;",
                                        "mpls-label": "Swap 16061",
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfadbd4",
                                "nh-index": "616",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 16061  /52 -> {Swap 16061}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "16062",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:26"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "205",
                                "nh": [
                                    {
                                        "label-element": "0xc5f6c90",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "1",
                                        "load-balance-label": "Label 16062: None;",
                                        "mpls-label": "Swap 16062",
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xbb6e474",
                                "nh-index": "619",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 16062  /52 -> {Swap 16062}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "16063",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:15:26"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "label-element": "0xc5c8150",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "2",
                                        "load-balance-label": "Label 16063: None;",
                                        "mpls-label": "Swap 16063",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5f72d0",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "2",
                                        "label-ttl-action": "no-prop-ttl, no-prop-ttl, no-prop-ttl(top)",
                                        "load-balance-label": "Label 16063: None; Label 1650: None; Label 1913: None;",
                                        "mpls-label": "Swap 16063, Push 1650, Push 1913(top)",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc14894",
                                "nh-index": "0",
                                "nh-reference-count": "1",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 16063  /52 -> {list:Swap 16063, Swap 16063, Push 1650, Push 1913(top)}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "16072",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:56"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "5",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8a8",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "140",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xe336114",
                                "nh-index": "622",
                                "nh-reference-count": "6",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 16072  /52 -> {Pop      }"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "16072(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:56"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "5",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8d0",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "140",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xe34b8f4",
                                "nh-index": "624",
                                "nh-reference-count": "6",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 16072  /56 -> {Pop      }"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "16073",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w5d 22:11:26"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1001",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8a8",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5ee928",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "3",
                                        "load-balance-label": "Label 16073: None;",
                                        "mpls-label": "Swap 16073",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc16b94",
                                "nh-index": "0",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 16073  /52 -> {list:Pop      , Swap 16073}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "16073(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w5d 22:11:26"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1001",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8d0",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5ee928",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "3",
                                        "load-balance-label": "Label 16073: None;",
                                        "mpls-label": "Swap 16073",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc16c14",
                                "nh-index": "0",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 16073  /56 -> {list:Pop      , Swap 16073}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "17000",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 5:30:11"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "label-element": "0xc5f6d30",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "2",
                                        "load-balance-label": "Label 17000: None;",
                                        "mpls-label": "Swap 17000",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5f34a0",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "2",
                                        "label-ttl-action": "no-prop-ttl, no-prop-ttl, no-prop-ttl(top)",
                                        "load-balance-label": "Label 17000: None; Label 1650: None; Label 1913: None;",
                                        "mpls-label": "Swap 17000, Push 1650, Push 1913(top)",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc14b94",
                                "nh-index": "0",
                                "nh-reference-count": "1",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 17000  /52 -> {list:Swap 17000, Swap 17000, Push 1650, Push 1913(top)}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "28985",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:56"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "0",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8a8",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "140",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xe336114",
                                "nh-index": "622",
                                "nh-reference-count": "6",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 28985  /52 -> {Pop      }"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "28985(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:56"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "0",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8d0",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "140",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xe34b8f4",
                                "nh-index": "624",
                                "nh-reference-count": "6",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 28985  /56 -> {Pop      }"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "28986",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:56"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "0",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8a8",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "140",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xe336114",
                                "nh-index": "622",
                                "nh-reference-count": "6",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 28986  /52 -> {Pop      }"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "28986(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:56"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "0",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8d0",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "140",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xe34b8f4",
                                "nh-index": "624",
                                "nh-reference-count": "6",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 28986  /56 -> {Pop      }"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "167966",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w5d 22:11:26"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "0",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8a8",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5ee928",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "3",
                                        "load-balance-label": "Label 16073: None;",
                                        "mpls-label": "Swap 16073",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc16b94",
                                "nh-index": "0",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 167966 /52 -> {list:Pop      , Swap 16073}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "167966(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w5d 22:11:26"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "0",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8d0",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0",
                                        "weight": "0x1"
                                    },
                                    {
                                        "label-element": "0xc5ee928",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "3",
                                        "load-balance-label": "Label 16073: None;",
                                        "mpls-label": "Swap 16073",
                                        "nh-string": "Next hop",
                                        "session": "0",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0xf000"
                                    }
                                ],
                                "nh-address": "0xbc16c14",
                                "nh-index": "0",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 167966 /56 -> {list:Pop      , Swap 16073}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "167967",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w5d 22:11:35"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "0",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8a8",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "1ae",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdbd53f4",
                                "nh-index": "655",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 167967 /52 -> {Pop      }"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "167967(S=0)",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w5d 22:11:35"
                                },
                                "announce-bits": "1",
                                "announce-tasks": "1-KRT",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "0",
                                "nh": [
                                    {
                                        "label-element": "0xc5cd8d0",
                                        "label-element-childcount": "0",
                                        "label-element-lspid": "0",
                                        "label-element-parent": "0x0",
                                        "label-element-refcount": "7",
                                        "load-balance-label": "None;",
                                        "mpls-label": "Pop",
                                        "nh-string": "Next hop",
                                        "session": "1ae",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdbd5334",
                                "nh-index": "656",
                                "nh-reference-count": "2",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "5",
                                "protocol-name": "L-OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 167967 /56 -> {Pop      }"
                            }
                        }
                    ],
                    "table-name": "mpls.0",
                    "total-route-count": "44"
                },
                {
                    "active-route-count": "22",
                    "destination-count": "22",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "::/0",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:55"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 3-LDP 5-Resolve tree 5",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "101",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "144",
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "nh-address": "0xe34bb94",
                                "nh-index": "628",
                                "nh-reference-count": "19",
                                "nh-type": "Router",
                                "preference": "150",
                                "protocol-name": "OSPF3",
                                "rt-entry-state": "Active Int Ext",
                                "rt-tag": "0",
                                "task-name": "OSPF3",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel ::/0 -> {fe80::250:56ff:fe8d:72bd}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2001:db8:6aa8:6a53::1001/128",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:55"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 3-LDP 5-Resolve tree 5",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "150",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "144",
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "nh-address": "0xe34bb94",
                                "nh-index": "628",
                                "nh-reference-count": "19",
                                "nh-type": "Router",
                                "preference": "150",
                                "protocol-name": "OSPF3",
                                "rt-entry-state": "Active Int Ext",
                                "rt-tag": "0",
                                "task-name": "OSPF3",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 2001:db8:6aa8:6a53::1001/128 -> {fe80::250:56ff:fe8d:72bd}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2001:db8:b0f8:ca45::13/128",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:17"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 3-LDP 5-Resolve tree 5",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "200",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "144",
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "nh-address": "0xe34bb94",
                                "nh-index": "628",
                                "nh-reference-count": "19",
                                "nh-type": "Router",
                                "preference": "10",
                                "protocol-name": "OSPF3",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF3",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 2001:db8:b0f8:ca45::13/128 -> {fe80::250:56ff:fe8d:72bd}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2001:db8:b0f8:ca45::14/128",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:17"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 3-LDP 5-Resolve tree 5",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "205",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "144",
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "nh-address": "0xe34bb94",
                                "nh-index": "628",
                                "nh-reference-count": "19",
                                "nh-type": "Router",
                                "preference": "10",
                                "protocol-name": "OSPF3",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF3",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 2001:db8:b0f8:ca45::14/128 -> {fe80::250:56ff:fe8d:72bd}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2001:db8:b0f8:3ab::/64",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:17"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 3-LDP 5-Resolve tree 5",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "205",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "144",
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "nh-address": "0xe34bb94",
                                "nh-index": "628",
                                "nh-reference-count": "19",
                                "nh-type": "Router",
                                "preference": "10",
                                "protocol-name": "OSPF3",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF3",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 2001:db8:b0f8:3ab::/64 -> {fe80::250:56ff:fe8d:72bd}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2001:db8:eb18:ca45::1/128",
                            "rt-entry": {
                                "age": {
                                    "#text": "3w0d 18:21:55"
                                },
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "inactive-reason": "Route Preference",
                                "local-as": "65171",
                                "metric": "100",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "144",
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "nh-address": "0xe34bb94",
                                "nh-index": "628",
                                "nh-reference-count": "19",
                                "nh-type": "Router",
                                "preference": "10",
                                "protocol-name": "OSPF3",
                                "rt-entry-state": "Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF3",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "2",
                                "@junos:format": "2 entries"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 2001:db8:eb18:ca45::1/128 -> {2001:db8:eb18:6337::1}\nOSPF3 realm ipv6-unicast area : 0.0.0.0, LSA ID : 0.0.0.1, LSA type : Extern"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2001:db8:eb18:ca45::2/128",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:55"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 3-LDP 5-Resolve tree 5",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "105",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "144",
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "nh-address": "0xe34bb94",
                                "nh-index": "628",
                                "nh-reference-count": "19",
                                "nh-type": "Router",
                                "preference": "10",
                                "protocol-name": "OSPF3",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF3",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 2001:db8:eb18:ca45::2/128 -> {fe80::250:56ff:fe8d:72bd}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2001:db8:eb18:e26e::/64",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:21:55"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 3-LDP 5-Resolve tree 5",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "105",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "144",
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "nh-address": "0xe34bb94",
                                "nh-index": "628",
                                "nh-reference-count": "19",
                                "nh-type": "Router",
                                "preference": "10",
                                "protocol-name": "OSPF3",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF3",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 2001:db8:eb18:e26e::/64 -> {fe80::250:56ff:fe8d:72bd}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2001:db8:eb18:f5e6::/64",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 5:30:12"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 3-LDP 5-Resolve tree 5",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "225",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "144",
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "nh-address": "0xe34bb94",
                                "nh-index": "628",
                                "nh-reference-count": "19",
                                "nh-type": "Router",
                                "preference": "10",
                                "protocol-name": "OSPF3",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF3",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 2001:db8:eb18:f5e6::/64 -> {fe80::250:56ff:fe8d:72bd}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2001:db8:eb18:6d57::/64",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:22:00"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 3-LDP 5-Resolve tree 5",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "125",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "142",
                                        "to": "fe80::250:56ff:fe8d:53c0",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "nh-address": "0xdfa4454",
                                "nh-index": "621",
                                "nh-reference-count": "4",
                                "nh-type": "Router",
                                "preference": "10",
                                "protocol-name": "OSPF3",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF3",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 2001:db8:eb18:6d57::/64 -> {fe80::250:56ff:fe8d:53c0}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2001:db8:eb18:9627::/64",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:10:17"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 3-LDP 5-Resolve tree 5",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "200",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "144",
                                        "to": "fe80::250:56ff:fe8d:72bd",
                                        "via": "ge-0/0/1.0"
                                    }
                                ],
                                "nh-address": "0xe34bb94",
                                "nh-index": "628",
                                "nh-reference-count": "19",
                                "nh-type": "Router",
                                "preference": "10",
                                "protocol-name": "OSPF3",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF3",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 2001:db8:eb18:9627::/64 -> {fe80::250:56ff:fe8d:72bd}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "2001:db8:223c:ca45::c/128",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:22:00"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 3-LDP 5-Resolve tree 5",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "5",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "142",
                                        "to": "fe80::250:56ff:fe8d:53c0",
                                        "via": "ge-0/0/0.0"
                                    }
                                ],
                                "nh-address": "0xdfa4454",
                                "nh-index": "621",
                                "nh-reference-count": "4",
                                "nh-type": "Router",
                                "preference": "10",
                                "protocol-name": "OSPF3",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF3",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 2001:db8:223c:ca45::c/128 -> {fe80::250:56ff:fe8d:53c0}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "ff02::5/128",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "29w5d 23:06:46"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 3-LDP 5-Resolve tree 5",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1",
                                "nh-address": "0xbb66cd4",
                                "nh-index": "0",
                                "nh-reference-count": "9",
                                "nh-type": "MultiRecv",
                                "preference": "10",
                                "protocol-name": "OSPF3",
                                "rt-entry-state": "Active NoReadvrt Int",
                                "task-name": "OSPF3 I/O./var/run/ppmd_control",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel ff02::5/128 -> {}"
                            }
                        }
                    ],
                    "table-name": "inet6.0",
                    "total-route-count": "23"
                }
            ]
        }
    }

    golden_output_2 = {'execute.return_value': '''
        show route protocol ospf table inet.0 extensive

        inet.0: 929 destinations, 1615 routes (929 active, 0 holddown, 0 hidden)
        0.0.0.0/0 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 0.0.0.0/0 -> {10.169.14.121}
                *OSPF   Preference: 150/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int Ext>
                        Local AS: 65171 
                        Age: 3w2d 4:46:58  Metric: 101 
                        Validation State: unverified 
                            Tag: 0 
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.1.0.0/24 (2 entries, 1 announced)
                State: <FlashAll>
                OSPF   Preference: 150/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Int Ext>
                        Inactive reason: Route Preference
                        Local AS: 65171 
                        Age: 3w2d 4:46:58  Metric: 20 
                        Validation State: unverified 
                            Tag: 0 
                        Task: OSPF
                        AS path: I 

        10.36.3.3/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.36.3.3/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.16.0.0/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.16.0.0/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:13:49  Metric: 1200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.100.5.5/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.100.5.5/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:13:49  Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        10.19.198.28/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.19.198.28/30 -> {10.189.5.94}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 614
                        Address: 0xdbe48d4
                        Next-hop reference count: 6
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0x1, selected
                        Session Id: 0x140
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:25:19      Metric: 1005 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.19.198.239/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.19.198.239/32 -> {10.19.198.26}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 605
                        Address: 0xdbc0d54
                        Next-hop reference count: 4
                        Next hop: 10.19.198.26 via ge-0/0/2.0 weight 0x1, selected
                        Session Id: 0x1ae
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 1w5d 22:14:58      Metric: 1001 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        10.174.132.237/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.174.132.237/32 -> {10.169.14.121}
                *OSPF   Preference: 150/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int Ext>
                        Local AS: 65171 
                        Age: 3w2d 4:46:58  Metric: 150 
                        Validation State: unverified 
                            Tag: 0 
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.34.2.200/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.34.2.200/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:13:49  Metric: 205 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.34.2.250/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.34.2.250/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:13:49  Metric: 200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        10.34.2.251/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.34.2.251/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:13:49  Metric: 205 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        10.15.0.0/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.15.0.0/30 -> {10.19.198.26}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 605
                        Address: 0xdbc0d54
                        Next-hop reference count: 4
                        Next hop: 10.19.198.26 via ge-0/0/2.0 weight 0x1, selected
                        Session Id: 0x1ae
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 1w5d 22:14:58      Metric: 1001 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.64.0.0/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.64.0.0/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.196.212/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.196.212/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:13:49  Metric: 1200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.196.216/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.196.216/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:13:49  Metric: 1205 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.196.241/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.196.241/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        10.169.14.16/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.16/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:46:58  Metric: 105 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.14.32/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.32/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 5:33:46  Metric: 225 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.14.128/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.128/30 -> {10.189.5.94}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 614
                        Address: 0xdbe48d4
                        Next-hop reference count: 6
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0x1, selected
                        Session Id: 0x140
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:25:19      Metric: 125 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.14.156/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.156/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:13:51  Metric: 200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.14.240/32 (2 entries, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.240/32 -> {10.169.14.121}
        OSPF area : 0.0.0.0, LSA ID : 10.169.14.240, LSA type : Extern
                OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Int>
                        Inactive reason: Route Preference
                        Local AS: 65171 
                        Age: 3w2d 4:46:58  Metric: 100 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        AS path: I 

        10.169.14.241/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.241/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:46:58  Metric: 105 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        10.169.14.242/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.242/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:46:58  Metric: 100 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.169.14.243/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.14.243/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:46:58  Metric: 105 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        10.189.5.253/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.189.5.253/32 -> {10.189.5.94}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 614
                        Address: 0xdbe48d4
                        Next-hop reference count: 6
                        Next hop: 10.189.5.94 via ge-0/0/0.0 weight 0x1, selected
                        Session Id: 0x140
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w0d 18:25:19      Metric: 5 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 

        192.168.220.0/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.0/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 2w6d 6:13:49  Metric: 1200 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.0/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.0/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.1/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.1/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.2/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.2/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.3/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.3/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.4/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.4/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.5/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.5/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.6/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.6/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.7/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.7/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.8/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.8/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.9/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.9/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.10/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.10/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.11/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.11/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.12/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.12/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.13/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.13/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.14/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.14/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.15/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.15/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.16/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.16/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.17/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.17/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.18/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.18/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.19/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.19/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.20/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.20/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.21/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.21/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.22/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.22/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.23/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.23/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.24/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.24/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.25/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.25/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.26/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.26/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.27/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.27/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.28/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.28/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.29/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.29/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.30/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.30/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.31/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.31/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.32/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.32/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.33/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.33/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.34/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.34/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.35/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.35/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.36/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.36/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.37/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.37/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.38/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.38/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.39/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.39/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.40/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.40/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.41/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.41/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.42/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.42/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.43/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.43/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.44/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.44/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.45/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.45/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.46/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.46/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.47/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.47/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.48/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.48/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.49/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.49/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.50/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.50/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.51/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.51/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.52/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.52/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.53/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.53/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.54/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.54/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.55/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.55/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.56/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.56/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.57/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.57/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.58/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.58/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.59/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.59/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.60/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.60/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.61/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.61/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.62/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.62/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.63/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.63/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.64/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.64/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.65/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.65/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.66/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.66/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.67/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.67/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.68/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.68/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.69/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.69/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.70/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.70/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.71/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.71/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.72/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.72/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.73/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.73/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.74/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.74/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.75/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.75/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.76/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.76/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.77/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.77/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.78/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.78/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.79/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.79/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.80/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.80/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.81/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.81/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.82/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.82/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.83/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.83/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.84/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.84/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.85/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.85/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.86/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.86/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.87/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.87/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.88/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.88/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.89/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.89/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.90/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.90/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.91/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.91/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.92/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.92/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.93/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.93/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.94/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.94/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.95/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.95/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.96/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.96/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.97/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.97/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.98/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.98/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.99/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.99/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.100/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.100/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.101/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.101/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.102/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.102/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.103/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.103/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.104/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.104/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.105/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.105/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.106/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.106/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.107/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.107/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.108/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.108/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.109/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.109/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.110/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.110/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.111/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.111/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.112/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.112/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.113/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.113/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.114/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.114/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.115/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.115/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.116/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.116/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.117/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.117/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.118/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.118/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.119/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.119/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.120/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.120/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.121/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.121/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.122/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.122/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.123/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.123/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.124/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.124/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.125/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.125/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.126/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.126/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.127/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.127/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.128/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.128/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.129/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.129/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.130/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.130/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.131/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.131/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.132/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.132/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.133/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.133/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.134/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.134/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.135/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.135/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.136/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.136/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.137/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.137/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.138/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.138/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.139/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.139/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.140/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.140/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.141/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.141/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.142/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.142/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.143/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.143/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.144/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.144/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.145/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.145/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.146/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.146/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.147/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.147/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.148/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.148/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.149/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.149/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.150/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.150/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.151/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.151/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.152/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.152/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.153/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.153/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.154/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.154/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.155/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.155/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.156/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.156/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.157/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.157/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.158/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.158/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.159/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.159/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.160/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.160/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.161/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.161/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.162/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.162/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.163/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.163/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.164/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.164/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.165/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.165/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.166/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.166/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.167/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.167/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.168/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.168/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.169/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.169/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.170/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.170/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.171/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.171/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.172/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.172/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.173/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.173/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.174/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.174/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.175/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.175/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.176/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.176/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.177/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.177/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.178/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.178/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.179/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.179/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.180/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.180/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.181/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.181/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.182/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.182/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.183/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.183/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.184/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.184/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.185/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.185/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.186/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.186/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.187/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.187/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.188/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.188/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.189/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.189/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.190/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.190/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.191/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.191/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.192/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.192/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.193/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.193/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.194/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.194/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.195/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.195/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.196/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.196/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.197/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.197/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.198/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.198/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.199/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.199/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.220.200/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.220.200/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1202 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.111.0/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.111.0/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:54   Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.4.0/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.4.0/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:18:29   Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.100.0/25 (2 entries, 1 announced)
        TSI:
        KRT in-kernel 192.168.100.0/25 -> {10.169.14.121}
                *OSPF   Preference: 150/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int Ext>
                        Local AS: 65171 
                        Age: 2w5d 16:22:08      Metric: 32000 
                        Validation State: unverified 
                            Tag: 65000500 
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.100.252/32 (2 entries, 1 announced)
        TSI:
        KRT in-kernel 192.168.100.252/32 -> {10.169.14.121}
                *OSPF   Preference: 150/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int Ext>
                        Local AS: 65171 
                        Age: 2w5d 16:22:08      Metric: 32000 
                        Validation State: unverified 
                            Tag: 65000500 
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.36.48/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.36.48/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:46:58  Metric: 10100 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.36.56/30 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.36.56/30 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:46:58  Metric: 10100 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.36.119/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.36.119/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:46:58  Metric: 10101 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        192.168.36.120/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 192.168.36.120/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 3w2d 4:46:58  Metric: 10101 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 

        224.0.0.5/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 224.0.0.5/32 -> {}
                *OSPF   Preference: 10
                        Next hop type: MultiRecv, Next hop index: 0
                        Address: 0xbb66cd4
                        Next-hop reference count: 9
                        State: <Active NoReadvrt Int>
                        Local AS: 65171 
                        Age: 29w5d 23:10:09     Metric: 1 
                        Validation State: unverified 
                        Task: OSPF I/O./var/run/ppmd_control
                        Announcement bits (3): 0-KRT 5-LDP 7-Resolve tree 3 
                        AS path: I 
    '''}

    golden_parsed_output_2 = {
        "route-information": {
            "route-table": [
                {
                    "active-route-count": "929",
                    "destination-count": "929",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "0.0.0.0/0",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:46:58"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "101",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int Ext",
                                "rt-tag": "0",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 0.0.0.0/0 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.1.0.0/24",
                            "rt-entry": {
                                "age": {
                                    "#text": "3w2d 4:46:58"
                                },
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "inactive-reason": "Route Preference",
                                "local-as": "65171",
                                "metric": "20",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Int Ext",
                                "rt-tag": "0",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "2",
                                "@junos:format": "2 entries"
                            },
                            "rt-state": "FlashAll"
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.36.3.3/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.36.3.3/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.16.0.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:13:49"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1200",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.16.0.0/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.100.5.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:13:49"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.100.5.5/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.19.198.28/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:25:19"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1005",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "140",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdbe48d4",
                                "nh-index": "614",
                                "nh-reference-count": "6",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.19.198.28/30 -> {10.189.5.94}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.19.198.239/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w5d 22:14:58"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1001",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "1ae",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdbc0d54",
                                "nh-index": "605",
                                "nh-reference-count": "4",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.19.198.239/32 -> {10.19.198.26}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.174.132.237/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:46:58"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "150",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int Ext",
                                "rt-tag": "0",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.174.132.237/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.34.2.200/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:13:49"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "205",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.34.2.200/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.34.2.250/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:13:49"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "200",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.34.2.250/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.34.2.251/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:13:49"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "205",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.34.2.251/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.15.0.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "1w5d 22:14:58"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1001",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "1ae",
                                        "to": "10.19.198.26",
                                        "via": "ge-0/0/2.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdbc0d54",
                                "nh-index": "605",
                                "nh-reference-count": "4",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.15.0.0/30 -> {10.19.198.26}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.64.0.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.64.0.0/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.196.212/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:13:49"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1200",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.196.212/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.196.216/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:13:49"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1205",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.196.216/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.196.241/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.196.241/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.16/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:46:58"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "105",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.16/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.32/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 5:33:46"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "225",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.32/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.128/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:25:19"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "125",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "140",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdbe48d4",
                                "nh-index": "614",
                                "nh-reference-count": "6",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.128/30 -> {10.189.5.94}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.156/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:13:51"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "200",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.156/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.240/32",
                            "rt-entry": {
                                "age": {
                                    "#text": "3w2d 4:46:58"
                                },
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "inactive-reason": "Route Preference",
                                "local-as": "65171",
                                "metric": "100",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "2",
                                "@junos:format": "2 entries"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.240/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.241/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:46:58"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "105",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.241/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.242/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:46:58"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "100",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.242/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.14.243/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:46:58"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "105",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.14.243/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.189.5.253/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w0d 18:25:19"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "5",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "140",
                                        "to": "10.189.5.94",
                                        "via": "ge-0/0/0.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdbe48d4",
                                "nh-index": "614",
                                "nh-reference-count": "6",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.189.5.253/32 -> {10.189.5.94}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w6d 6:13:49"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1200",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.0/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.0/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.0/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.1/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.1/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.2/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.2/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.3/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.3/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.4/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.4/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.5/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.6/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.6/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.7/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.7/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.8/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.8/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.9/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.9/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.10/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.10/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.11/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.11/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.12/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.12/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.13/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.13/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.14/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.14/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.15/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.15/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.16/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.16/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.17/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.17/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.18/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.18/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.19/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.19/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.20/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.20/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.21/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.21/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.22/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.22/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.23/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.23/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.24/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.24/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.25/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.25/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.26/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.26/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.27/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.27/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.28/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.28/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.29/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.29/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.30/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.30/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.31/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.31/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.32/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.32/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.33/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.33/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.34/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.34/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.35/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.35/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.36/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.36/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.37/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.37/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.38/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.38/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.39/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.39/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.40/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.40/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.41/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.41/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.42/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.42/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.43/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.43/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.44/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.44/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.45/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.45/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.46/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.46/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.47/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.47/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.48/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.48/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.49/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.49/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.50/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.50/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.51/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.51/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.52/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.52/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.53/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.53/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.54/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.54/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.55/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.55/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.56/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.56/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.57/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.57/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.58/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.58/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.59/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.59/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.60/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.60/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.61/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.61/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.62/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.62/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.63/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.63/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.64/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.64/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.65/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.65/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.66/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.66/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.67/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.67/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.68/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.68/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.69/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.69/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.70/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.70/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.71/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.71/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.72/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.72/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.73/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.73/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.74/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.74/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.75/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.75/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.76/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.76/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.77/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.77/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.78/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.78/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.79/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.79/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.80/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.80/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.81/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.81/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.82/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.82/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.83/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.83/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.84/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.84/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.85/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.85/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.86/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.86/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.87/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.87/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.88/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.88/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.89/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.89/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.90/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.90/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.91/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.91/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.92/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.92/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.93/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.93/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.94/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.94/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.95/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.95/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.96/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.96/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.97/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.97/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.98/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.98/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.99/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.99/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.100/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.100/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.101/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.101/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.102/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.102/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.103/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.103/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.104/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.104/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.105/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.105/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.106/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.106/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.107/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.107/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.108/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.108/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.109/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.109/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.110/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.110/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.111/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.111/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.112/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.112/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.113/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.113/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.114/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.114/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.115/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.115/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.116/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.116/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.117/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.117/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.118/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.118/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.119/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.119/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.120/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.120/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.121/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.121/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.122/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.122/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.123/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.123/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.124/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.124/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.125/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.125/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.126/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.126/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.127/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.127/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.128/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.128/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.129/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.129/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.130/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.130/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.131/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.131/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.132/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.132/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.133/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.133/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.134/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.134/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.135/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.135/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.136/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.136/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.137/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.137/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.138/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.138/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.139/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.139/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.140/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.140/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.141/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.141/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.142/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.142/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.143/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.143/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.144/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.144/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.145/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.145/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.146/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.146/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.147/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.147/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.148/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.148/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.149/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.149/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.150/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.150/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.151/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.151/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.152/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.152/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.153/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.153/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.154/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.154/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.155/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.155/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.156/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.156/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.157/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.157/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.158/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.158/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.159/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.159/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.160/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.160/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.161/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.161/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.162/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.162/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.163/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.163/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.164/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.164/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.165/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.165/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.166/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.166/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.167/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.167/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.168/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.168/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.169/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.169/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.170/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.170/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.171/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.171/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.172/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.172/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.173/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.173/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.174/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.174/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.175/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.175/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.176/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.176/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.177/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.177/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.178/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.178/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.179/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.179/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.180/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.180/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.181/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.181/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.182/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.182/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.183/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.183/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.184/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.184/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.185/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.185/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.186/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.186/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.187/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.187/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.188/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.188/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.189/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.189/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.190/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.190/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.191/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.191/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.192/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.192/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.193/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.193/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.194/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.194/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.195/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.195/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.196/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.196/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.197/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.197/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.198/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.198/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.199/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.199/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.220.200/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1202",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.220.200/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.111.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:54"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.111.0/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.4.0/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:18:29"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.4.0/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.100.0/25",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w5d 16:22:08"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "32000",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int Ext",
                                "rt-tag": "65000500",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "2",
                                "@junos:format": "2 entries"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.100.0/25 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.100.252/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "2w5d 16:22:08"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "32000",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "150",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int Ext",
                                "rt-tag": "65000500",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "2",
                                "@junos:format": "2 entries"
                            },
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.100.252/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.36.48/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:46:58"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "10100",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.36.48/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.36.56/30",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:46:58"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "10100",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.36.56/30 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.36.119/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:46:58"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "10101",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.36.119/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "192.168.36.120/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "3w2d 4:46:58"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "10101",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 192.168.36.120/32 -> {10.169.14.121}"
                            }
                        },
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "224.0.0.5/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "29w5d 23:10:09"
                                },
                                "announce-bits": "3",
                                "announce-tasks": "0-KRT 5-LDP 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1",
                                "nh-address": "0xbb66cd4",
                                "nh-index": "0",
                                "nh-reference-count": "9",
                                "nh-type": "MultiRecv",
                                "preference": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active NoReadvrt Int",
                                "task-name": "OSPF I/O./var/run/ppmd_control",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 224.0.0.5/32 -> {}"
                            }
                        }
                    ],
                    "table-name": "inet.0",
                    "total-route-count": "1615"
                }
            ]
        }
    }

    golden_output_3 = {'execute.return_value': '''
        show route protocol ospf table inet.0 extensive 10.169.196.241

        inet.0: 929 destinations, 1615 routes (929 active, 0 holddown, 0 hidden)
        10.169.196.241/32 (1 entry, 1 announced)
                State: <FlashAll>
        TSI:
        KRT in-kernel 10.169.196.241/32 -> {10.169.14.121}
                *OSPF   Preference: 10/10
                        Next hop type: Router, Next hop index: 613
                        Address: 0xdfa7934
                        Next-hop reference count: 458
                        Next hop: 10.169.14.121 via ge-0/0/1.0 weight 0x1, selected
                        Session Id: 0x141
                        State: <Active Int>
                        Local AS: 65171 
                        Age: 6d 17:23:01 	Metric: 1201 
                        Validation State: unverified 
                        Area: 0.0.0.8
                        Task: OSPF
                        Announcement bits (2): 0-KRT 7-Resolve tree 3 
                        AS path: I 
    '''}

    golden_parsed_output_3 = {
        "route-information": {
            "route-table": [
                {
                    "active-route-count": "929",
                    "destination-count": "929",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [
                        {
                            "rt-announced-count": "1",
                            "rt-destination": "10.169.196.241/32",
                            "rt-entry": {
                                "active-tag": "*",
                                "age": {
                                    "#text": "6d 17:23:01"
                                },
                                "announce-bits": "2",
                                "announce-tasks": "0-KRT 7-Resolve tree 3",
                                "as-path": "AS path: I",
                                "bgp-path-attributes": {
                                    "attr-as-path-effective": {
                                        "aspath-effective-string": "AS path:",
                                        "attr-value": "I"
                                    }
                                },
                                "local-as": "65171",
                                "metric": "1201",
                                "nh": [
                                    {
                                        "nh-string": "Next hop",
                                        "session": "141",
                                        "to": "10.169.14.121",
                                        "via": "ge-0/0/1.0",
                                        "weight": "0x1"
                                    }
                                ],
                                "nh-address": "0xdfa7934",
                                "nh-index": "613",
                                "nh-reference-count": "458",
                                "nh-type": "Router",
                                "preference": "10",
                                "preference2": "10",
                                "protocol-name": "OSPF",
                                "rt-entry-state": "Active Int",
                                "rt-ospf-area": "0.0.0.8",
                                "task-name": "OSPF",
                                "validation-state": "unverified"
                            },
                            "rt-entry-count": {
                                "#text": "1",
                                "@junos:format": "1 entry"
                            },
                            "rt-state": "FlashAll",
                            "tsi": {
                                "#text": "KRT in-kernel 10.169.196.241/32 -> {10.169.14.121}"
                            }
                        }
                    ],
                    "table-name": "inet.0",
                    "total-route-count": "1615"
                }
            ]
        }
    }

    def test_empty(self):
        self.device = Mock(**self.empty_output)
        obj = ShowRouteProtocolExtensive(device=self.device)
        with self.assertRaises(SchemaEmptyParserError):
            parsed_output = obj.parse(
                protocol='ospf')

    def test_golden(self):
        self.device = Mock(**self.golden_output)
        obj = ShowRouteProtocolExtensive(device=self.device)
        parsed_output = obj.parse(
            protocol='ospf')
        self.assertEqual(parsed_output, self.golden_parsed_output)

    def test_golden_2(self):
        self.device = Mock(**self.golden_output_2)
        obj = ShowRouteProtocolExtensive(device=self.device)
        parsed_output = obj.parse(
            protocol='ospf',
            table='inet.0')
        self.assertEqual(parsed_output, self.golden_parsed_output_2)
    
    def test_golden_3(self):
        self.device = Mock(**self.golden_output_3)
        obj = ShowRouteProtocolExtensive(device=self.device)
        parsed_output = obj.parse(
            protocol='ospf',
            table='inet.0',
            destination='10.169.196.241')
        self.assertEqual(parsed_output, self.golden_parsed_output_3)

'''
Unit test for:
    * show route forwarding-table summary
'''
class TestShowRouteForwardingTableSummary(unittest.TestCase):
    device = Device(name='aDevice')
    maxDiff = None

    empty_output = {'execute.return_value': ''}

    golden_output = {'execute.return_value': '''
        show route forwarding-table summary
        Routing table: default.inet
        Internet:
        Enabled protocols: Bridging, 
                user:         918 routes
                perm:          5 routes
                intf:         11 routes
                dest:         21 routes

        Routing table: __pfe_private__.inet
        Internet:
        Enabled protocols: Bridging, 
                perm:          5 routes

        Routing table: __juniper_services__.inet
        Internet:
        Enabled protocols: Bridging, 
                perm:          5 routes
                intf:          2 routes
                dest:          3 routes

        Routing table: __master.anon__.inet
        Internet:
        Enabled protocols: Bridging, Dual VLAN, 
                perm:          5 routes

        Routing table: default.iso
        ISO:
        Enabled protocols: Bridging, 
                perm:          1 routes

        Routing table: __master.anon__.iso
        ISO:
        Enabled protocols: Bridging, Dual VLAN, 
                perm:          1 routes

        Routing table: default.inet6
        Internet6:
        Enabled protocols: Bridging, 
                user:         14 routes
                perm:          6 routes
                intf:         18 routes
                dest:          7 routes

        Routing table: __master.anon__.inet6
        Internet6:
        Enabled protocols: Bridging, Dual VLAN, 
                perm:          6 routes

        Routing table: default.mpls
        MPLS:
                user:         44 routes
                perm:          1 routes

        Routing table: __mpls-oam__.mpls
        MPLS:
        Enabled protocols: Bridging, Single VLAN, Dual VLAN, 
                perm:          1 routes

        Routing table: default-switch.bridge
        VPLS:
                perm:          1 routes

        Routing table: default.dhcp-snooping
        DHCP Snooping:
                perm:          1 routes
    '''}

    golden_parsed_output = {
        "forwarding-table-information": {
            "route-table": [
                {
                    "address-family": "Internet",
                    "enabled-protocols": "Bridging,",
                    "route-table-summary": [
                        {
                            "route-count": "918",
                            "route-table-type": "user"
                        },
                        {
                            "route-count": "5",
                            "route-table-type": "perm"
                        },
                        {
                            "route-count": "11",
                            "route-table-type": "intf"
                        },
                        {
                            "route-count": "21",
                            "route-table-type": "dest"
                        }
                    ],
                    "table-name": "default.inet"
                },
                {
                    "address-family": "Internet",
                    "enabled-protocols": "Bridging,",
                    "route-table-summary": [
                        {
                            "route-count": "5",
                            "route-table-type": "perm"
                        }
                    ],
                    "table-name": "__pfe_private__.inet"
                },
                {
                    "address-family": "Internet",
                    "enabled-protocols": "Bridging,",
                    "route-table-summary": [
                        {
                            "route-count": "5",
                            "route-table-type": "perm"
                        },
                        {
                            "route-count": "2",
                            "route-table-type": "intf"
                        },
                        {
                            "route-count": "3",
                            "route-table-type": "dest"
                        }
                    ],
                    "table-name": "__juniper_services__.inet"
                },
                {
                    "address-family": "Internet",
                    "enabled-protocols": "Bridging, Dual VLAN,",
                    "route-table-summary": [
                        {
                            "route-count": "5",
                            "route-table-type": "perm"
                        }
                    ],
                    "table-name": "__master.anon__.inet"
                },
                {
                    "address-family": "ISO",
                    "enabled-protocols": "Bridging,",
                    "route-table-summary": [
                        {
                            "route-count": "1",
                            "route-table-type": "perm"
                        }
                    ],
                    "table-name": "default.iso"
                },
                {
                    "address-family": "ISO",
                    "enabled-protocols": "Bridging, Dual VLAN,",
                    "route-table-summary": [
                        {
                            "route-count": "1",
                            "route-table-type": "perm"
                        }
                    ],
                    "table-name": "__master.anon__.iso"
                },
                {
                    "address-family": "Internet6",
                    "enabled-protocols": "Bridging,",
                    "route-table-summary": [
                        {
                            "route-count": "14",
                            "route-table-type": "user"
                        },
                        {
                            "route-count": "6",
                            "route-table-type": "perm"
                        },
                        {
                            "route-count": "18",
                            "route-table-type": "intf"
                        },
                        {
                            "route-count": "7",
                            "route-table-type": "dest"
                        }
                    ],
                    "table-name": "default.inet6"
                },
                {
                    "address-family": "Internet6",
                    "enabled-protocols": "Bridging, Dual VLAN,",
                    "route-table-summary": [
                        {
                            "route-count": "6",
                            "route-table-type": "perm"
                        }
                    ],
                    "table-name": "__master.anon__.inet6"
                },
                {
                    "address-family": "MPLS",
                    "route-table-summary": [
                        {
                            "route-count": "44",
                            "route-table-type": "user"
                        },
                        {
                            "route-count": "1",
                            "route-table-type": "perm"
                        }
                    ],
                    "table-name": "default.mpls"
                },
                {
                    "address-family": "MPLS",
                    "enabled-protocols": "Bridging, Single VLAN, Dual VLAN,",
                    "route-table-summary": [
                        {
                            "route-count": "1",
                            "route-table-type": "perm"
                        }
                    ],
                    "table-name": "__mpls-oam__.mpls"
                },
                {
                    "address-family": "VPLS",
                    "route-table-summary": [
                        {
                            "route-count": "1",
                            "route-table-type": "perm"
                        }
                    ],
                    "table-name": "default-switch.bridge"
                },
                {
                    "address-family": "DHCP Snooping",
                    "route-table-summary": [
                        {
                            "route-count": "1",
                            "route-table-type": "perm"
                        }
                    ],
                    "table-name": "default.dhcp-snooping"
                }
            ]
        }
    }

    def test_empty(self):
        self.device = Mock(**self.empty_output)
        obj = ShowRouteForwardingTableSummary(device=self.device)
        with self.assertRaises(SchemaEmptyParserError):
            parsed_output = obj.parse()

    def test_golden(self):
        self.device = Mock(**self.golden_output)
        obj = ShowRouteForwardingTableSummary(device=self.device)
        parsed_output = obj.parse()
        self.assertEqual(parsed_output, self.golden_parsed_output)

'''
Unit test for:
    * show route receive-protocol {protocol} {peer}
'''
class TestShowRouteReceiveProtocol(unittest.TestCase):
    device = Device(name='aDevice')
    maxDiff = None

    empty_output = {'execute.return_value': ''}

    golden_output = {'execute.return_value': '''
        show route receive-protocol bgp 10.189.5.253
        inet.0: 929 destinations, 1615 routes (929 active, 0 holddown, 0 hidden)
        Prefix      Nexthop        MED     Lclpref    AS path
        10.220.0.0/16           10.189.5.253         12003   120        (65151 65000) I
        10.229.0.0/16            10.189.5.253         12003   120        (65151 65000) I
        10.189.0.0/16            10.189.5.253         12003   120        (65151 65000) I
        10.151.0.0/16            10.189.5.253         12003   120        (65151 65000) I
        10.115.0.0/16            10.189.5.253         12003   120        (65151 65000) I
        10.81.0.0/16            10.189.5.253         12003   120        (65151 65000) I
        10.49.0.0/16            10.189.5.253         12003   120        (65151 65000) I
        10.19.0.0/16            10.189.5.253         12003   120        (65151 65000) I
        inet.3: 11 destinations, 11 routes (11 active, 0 holddown, 0 hidden)

        mpls.0: 44 destinations, 44 routes (44 active, 0 holddown, 0 hidden)

        inet6.0: 22 destinations, 23 routes (22 active, 0 holddown, 0 hidden)
    '''}

    golden_parsed_output = {
        "route-information": {
            "route-table": [
                {
                    "active-route-count": "929",
                    "destination-count": "929",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "rt": [
                        {
                            "rt-destination": "10.220.0.0/16",
                            "rt-entry": {
                                "as-path": "(65151 65000) I",
                                "local-preference": "120",
                                "med": "12003",
                                "nh": {
                                    "to": "10.189.5.253"
                                },
                                "protocol-name": "BGP"
                            }
                        },
                        {
                            "rt-destination": "10.229.0.0/16",
                            "rt-entry": {
                                "as-path": "(65151 65000) I",
                                "local-preference": "120",
                                "med": "12003",
                                "nh": {
                                    "to": "10.189.5.253"
                                },
                                "protocol-name": "BGP"
                            }
                        },
                        {
                            "rt-destination": "10.189.0.0/16",
                            "rt-entry": {
                                "as-path": "(65151 65000) I",
                                "local-preference": "120",
                                "med": "12003",
                                "nh": {
                                    "to": "10.189.5.253"
                                },
                                "protocol-name": "BGP"
                            }
                        },
                        {
                            "rt-destination": "10.151.0.0/16",
                            "rt-entry": {
                                "as-path": "(65151 65000) I",
                                "local-preference": "120",
                                "med": "12003",
                                "nh": {
                                    "to": "10.189.5.253"
                                },
                                "protocol-name": "BGP"
                            }
                        },
                        {
                            "rt-destination": "10.115.0.0/16",
                            "rt-entry": {
                                "as-path": "(65151 65000) I",
                                "local-preference": "120",
                                "med": "12003",
                                "nh": {
                                    "to": "10.189.5.253"
                                },
                                "protocol-name": "BGP"
                            }
                        },
                        {
                            "rt-destination": "10.81.0.0/16",
                            "rt-entry": {
                                "as-path": "(65151 65000) I",
                                "local-preference": "120",
                                "med": "12003",
                                "nh": {
                                    "to": "10.189.5.253"
                                },
                                "protocol-name": "BGP"
                            }
                        },
                        {
                            "rt-destination": "10.49.0.0/16",
                            "rt-entry": {
                                "as-path": "(65151 65000) I",
                                "local-preference": "120",
                                "med": "12003",
                                "nh": {
                                    "to": "10.189.5.253"
                                },
                                "protocol-name": "BGP"
                            }
                        },
                        {
                            "rt-destination": "10.19.0.0/16",
                            "rt-entry": {
                                "as-path": "(65151 65000) I",
                                "local-preference": "120",
                                "med": "12003",
                                "nh": {
                                    "to": "10.189.5.253"
                                },
                                "protocol-name": "BGP"
                            }
                        }
                    ],
                    "table-name": "inet.0",
                    "total-route-count": "1615"
                },
                {
                    "active-route-count": "11",
                    "destination-count": "11",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "table-name": "inet.3",
                    "total-route-count": "11"
                },
                {
                    "active-route-count": "44",
                    "destination-count": "44",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "table-name": "mpls.0",
                    "total-route-count": "44"
                },
                {
                    "active-route-count": "22",
                    "destination-count": "22",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "table-name": "inet6.0",
                    "total-route-count": "23"
                }
            ]
        }
    }

    def test_empty(self):
        self.device = Mock(**self.empty_output)
        obj = ShowRouteReceiveProtocol(device=self.device)
        with self.assertRaises(SchemaEmptyParserError):
            parsed_output = obj.parse(
                protocol='bgp',
                peer='10.189.5.253'
            )

    def test_golden(self):
        self.device = Mock(**self.golden_output)
        obj = ShowRouteReceiveProtocol(device=self.device)
        parsed_output = obj.parse(
            protocol='bgp',
            peer='10.189.5.253')
        self.assertEqual(parsed_output, self.golden_parsed_output)

'''
Unit test for:
    * show route advertising-protocol {protocol} {neighbor}
'''
class TestShowRouteAdvertisingProtocol(unittest.TestCase):
    device = Device(name='aDevice')
    maxDiff = None

    empty_output = {'execute.return_value': ''}

    golden_output = {'execute.return_value': '''
        show route advertising-protocol bgp 10.189.5.253

        inet.0: 929 destinations, 1615 routes (929 active, 0 holddown, 0 hidden)
        Prefix        Nexthop          MED     Lclpref    AS path
        * 10.220.0.0/16           Self                 12003   120        (65151 65000) I
        * 10.229.0.0/16            Self                 12003   120        (65151 65000) I
        * 10.189.0.0/16            Self                 12003   120        (65151 65000) I
        * 10.151.0.0/16            Self                 12003   120        (65151 65000) I
        * 10.115.0.0/16            Self                 12003   120        (65151 65000) I
    '''}

    golden_parsed_output = {
        "route-information": {
            "route-table": {
                "active-route-count": "929",
                "destination-count": "929",
                "hidden-route-count": "0",
                "holddown-route-count": "0",
                "rt": [
                    {
                        "rt-destination": "10.220.0.0/16",
                        "rt-entry": {
                            "active-tag": "*",
                            "as-path": "(65151 65000) I",
                            "bgp-metric-flags": "Nexthop Change",
                            "local-preference": "120",
                            "med": "12003",
                            "nh": {
                                "to": "Self"
                            },
                            "protocol-name": "BGP"
                        }
                    },
                    {
                        "rt-destination": "10.229.0.0/16",
                        "rt-entry": {
                            "active-tag": "*",
                            "as-path": "(65151 65000) I",
                            "bgp-metric-flags": "Nexthop Change",
                            "local-preference": "120",
                            "med": "12003",
                            "nh": {
                                "to": "Self"
                            },
                            "protocol-name": "BGP"
                        }
                    },
                    {
                        "rt-destination": "10.189.0.0/16",
                        "rt-entry": {
                            "active-tag": "*",
                            "as-path": "(65151 65000) I",
                            "bgp-metric-flags": "Nexthop Change",
                            "local-preference": "120",
                            "med": "12003",
                            "nh": {
                                "to": "Self"
                            },
                            "protocol-name": "BGP"
                        }
                    },
                    {
                        "rt-destination": "10.151.0.0/16",
                        "rt-entry": {
                            "active-tag": "*",
                            "as-path": "(65151 65000) I",
                            "bgp-metric-flags": "Nexthop Change",
                            "local-preference": "120",
                            "med": "12003",
                            "nh": {
                                "to": "Self"
                            },
                            "protocol-name": "BGP"
                        }
                    },
                    {
                        "rt-destination": "10.115.0.0/16",
                        "rt-entry": {
                            "active-tag": "*",
                            "as-path": "(65151 65000) I",
                            "bgp-metric-flags": "Nexthop Change",
                            "local-preference": "120",
                            "med": "12003",
                            "nh": {
                                "to": "Self"
                            },
                            "protocol-name": "BGP"
                        }
                    }
                ],
                "table-name": "inet.0",
                "total-route-count": "1615"
            }
        }
    }

    def test_empty(self):
        self.device = Mock(**self.empty_output)
        obj = ShowRouteAdvertisingProtocol(device=self.device)
        with self.assertRaises(SchemaEmptyParserError):
            parsed_output = obj.parse(
                protocol='bgp',
                neighbor='10.189.5.253'
            )

    def test_golden(self):
        self.device = Mock(**self.golden_output)
        obj = ShowRouteAdvertisingProtocol(device=self.device)
        parsed_output = obj.parse(
            protocol='bgp',
            neighbor='10.189.5.253')
        self.assertEqual(parsed_output, self.golden_parsed_output)

'''
Unit test for:
    * show route summary
'''
class TestShowRouteSummary(unittest.TestCase):
    device = Device(name='aDevice')
    maxDiff = None

    empty_output = {'execute.return_value': ''}

    golden_output = {'execute.return_value': '''
        show route summary
        Autonomous system number: 65171
        Router ID: 10.189.5.252

        inet.0: 929 destinations, 1615 routes (929 active, 0 holddown, 0 hidden)
                    Direct:      6 routes,      6 active
                    Local:      5 routes,      5 active
                        OSPF:    236 routes,    234 active
                        BGP:   1366 routes,    682 active
                    Static:      1 routes,      1 active
                        LDP:      1 routes,      1 active

        inet.3: 11 destinations, 11 routes (11 active, 0 holddown, 0 hidden)
                        BGP:      2 routes,      2 active
                        LDP:      1 routes,      1 active
                    L-OSPF:      8 routes,      8 active

        mpls.0: 44 destinations, 44 routes (44 active, 0 holddown, 0 hidden)
                        MPLS:      6 routes,      6 active
                        LDP:     15 routes,     15 active
                    L-OSPF:     23 routes,     23 active

        inet6.0: 22 destinations, 23 routes (22 active, 0 holddown, 0 hidden)
                    Direct:      4 routes,      4 active
                    Local:      4 routes,      4 active
                    OSPF3:     13 routes,     12 active
                    Static:      1 routes,      1 active
                    INET6:      1 routes,      1 active
    '''}

    golden_parsed_output = {
        "route-summary-information": {
            "as-number": "65171",
            "route-table": [
                {
                    "active-route-count": "929",
                    "destination-count": "929",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "protocols": [
                        {
                            "active-route-count": "6",
                            "protocol-name": "Direct",
                            "protocol-route-count": "6"
                        },
                        {
                            "active-route-count": "5",
                            "protocol-name": "Local",
                            "protocol-route-count": "5"
                        },
                        {
                            "active-route-count": "234",
                            "protocol-name": "OSPF",
                            "protocol-route-count": "236"
                        },
                        {
                            "active-route-count": "682",
                            "protocol-name": "BGP",
                            "protocol-route-count": "1366"
                        },
                        {
                            "active-route-count": "1",
                            "protocol-name": "Static",
                            "protocol-route-count": "1"
                        },
                        {
                            "active-route-count": "1",
                            "protocol-name": "LDP",
                            "protocol-route-count": "1"
                        }
                    ],
                    "table-name": "inet.0",
                    "total-route-count": "1615"
                },
                {
                    "active-route-count": "11",
                    "destination-count": "11",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "protocols": [
                        {
                            "active-route-count": "2",
                            "protocol-name": "BGP",
                            "protocol-route-count": "2"
                        },
                        {
                            "active-route-count": "1",
                            "protocol-name": "LDP",
                            "protocol-route-count": "1"
                        },
                        {
                            "active-route-count": "8",
                            "protocol-name": "L-OSPF",
                            "protocol-route-count": "8"
                        }
                    ],
                    "table-name": "inet.3",
                    "total-route-count": "11"
                },
                {
                    "active-route-count": "44",
                    "destination-count": "44",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "protocols": [
                        {
                            "active-route-count": "6",
                            "protocol-name": "MPLS",
                            "protocol-route-count": "6"
                        },
                        {
                            "active-route-count": "15",
                            "protocol-name": "LDP",
                            "protocol-route-count": "15"
                        },
                        {
                            "active-route-count": "23",
                            "protocol-name": "L-OSPF",
                            "protocol-route-count": "23"
                        }
                    ],
                    "table-name": "mpls.0",
                    "total-route-count": "44"
                },
                {
                    "active-route-count": "22",
                    "destination-count": "22",
                    "hidden-route-count": "0",
                    "holddown-route-count": "0",
                    "protocols": [
                        {
                            "active-route-count": "4",
                            "protocol-name": "Direct",
                            "protocol-route-count": "4"
                        },
                        {
                            "active-route-count": "4",
                            "protocol-name": "Local",
                            "protocol-route-count": "4"
                        },
                        {
                            "active-route-count": "12",
                            "protocol-name": "OSPF3",
                            "protocol-route-count": "13"
                        },
                        {
                            "active-route-count": "1",
                            "protocol-name": "Static",
                            "protocol-route-count": "1"
                        },
                        {
                            "active-route-count": "1",
                            "protocol-name": "INET6",
                            "protocol-route-count": "1"
                        }
                    ],
                    "table-name": "inet6.0",
                    "total-route-count": "23"
                }
            ],
            "router-id": "10.189.5.252"
        }
    }

    def test_empty(self):
        self.device = Mock(**self.empty_output)
        obj = ShowRouteSummary(device=self.device)
        with self.assertRaises(SchemaEmptyParserError):
            parsed_output = obj.parse()

    def test_golden(self):
        self.device = Mock(**self.golden_output)
        obj = ShowRouteSummary(device=self.device)
        parsed_output = obj.parse()
        self.assertEqual(parsed_output, self.golden_parsed_output)

'''
Unit test for:
    * show route instance detail
'''
class TestShowRouteInstanceDetail(unittest.TestCase):
    device = Device(name='aDevice')
    maxDiff = None

    empty_output = {'execute.return_value': ''}

    golden_output = {'execute.return_value': '''
        show route instance detail
        master:
        Router ID: 10.189.5.252
        Type: forwarding        State: Active        
        Tables:
            inet.0                 : 1615 routes (929 active, 0 holddown, 0 hidden)
            inet.3                 : 11 routes (11 active, 0 holddown, 0 hidden)
            mpls.0                 : 44 routes (44 active, 0 holddown, 0 hidden)
            inet6.0                : 23 routes (22 active, 0 holddown, 0 hidden)

        __juniper_private1__:
        Router ID: 0.0.0.0
        Type: forwarding        State: Active        
        Interfaces:
            pfh-0/0/0.16383
            pfe-0/0/0.16383
            lo0.16385
            em1.0
        Tables:
            __juniper_private1__.inet.0: 6 routes (5 active, 0 holddown, 0 hidden)
            __juniper_private1__.inet6.0: 3 routes (3 active, 0 holddown, 0 hidden)

        __juniper_private2__:
        Router ID: 0.0.0.0
        Type: forwarding        State: Active        
        Interfaces:
            pfh-0/0/0.16384
            lo0.16384
        Tables:
            __juniper_private2__.inet.0: 1 routes (0 active, 0 holddown, 1 hidden)

        __juniper_private4__:
        Router ID: 0.0.0.0
        Type: forwarding        State: Active        

        __master.anon__:
        Router ID: 0.0.0.0
        Type: forwarding        State: Active        

        mgmt_junos:
        Router ID: 0.0.0.0
        Type: forwarding        State: Active      
    '''}

    golden_parsed_output = {
        "instance-information": {
            "instance-core": [
                {
                    "instance-name": "master",
                    "instance-rib": [
                        {
                            "irib-active-count": "929",
                            "irib-hidden-count": "0",
                            "irib-holddown-count": "0",
                            "irib-name": "inet.0",
                            "irib-route-count": "1615"
                        },
                        {
                            "irib-active-count": "11",
                            "irib-hidden-count": "0",
                            "irib-holddown-count": "0",
                            "irib-name": "inet.3",
                            "irib-route-count": "11"
                        },
                        {
                            "irib-active-count": "44",
                            "irib-hidden-count": "0",
                            "irib-holddown-count": "0",
                            "irib-name": "mpls.0",
                            "irib-route-count": "44"
                        },
                        {
                            "irib-active-count": "22",
                            "irib-hidden-count": "0",
                            "irib-holddown-count": "0",
                            "irib-name": "inet6.0",
                            "irib-route-count": "23"
                        }
                    ],
                    "instance-state": "Active",
                    "instance-type": "forwarding",
                    "router-id": "10.189.5.252"
                },
                {
                    "instance-interface": [
                        {
                            "interface-name": "pfh-0/0/0.16383"
                        },
                        {
                            "interface-name": "pfe-0/0/0.16383"
                        },
                        {
                            "interface-name": "lo0.16385"
                        },
                        {
                            "interface-name": "em1.0"
                        }
                    ],
                    "instance-name": "__juniper_private1__",
                    "instance-rib": [
                        {
                            "irib-active-count": "5",
                            "irib-hidden-count": "0",
                            "irib-holddown-count": "0",
                            "irib-name": "__juniper_private1__.inet.0",
                            "irib-route-count": "6"
                        },
                        {
                            "irib-active-count": "3",
                            "irib-hidden-count": "0",
                            "irib-holddown-count": "0",
                            "irib-name": "__juniper_private1__.inet6.0",
                            "irib-route-count": "3"
                        }
                    ],
                    "instance-state": "Active",
                    "instance-type": "forwarding",
                    "router-id": "0.0.0.0"
                },
                {
                    "instance-interface": [
                        {
                            "interface-name": "pfh-0/0/0.16384"
                        },
                        {
                            "interface-name": "lo0.16384"
                        }
                    ],
                    "instance-name": "__juniper_private2__",
                    "instance-rib": [
                        {
                            "irib-active-count": "0",
                            "irib-hidden-count": "1",
                            "irib-holddown-count": "0",
                            "irib-name": "__juniper_private2__.inet.0",
                            "irib-route-count": "1"
                        }
                    ],
                    "instance-state": "Active",
                    "instance-type": "forwarding",
                    "router-id": "0.0.0.0"
                },
                {
                    "instance-name": "__juniper_private4__",
                    "instance-state": "Active",
                    "instance-type": "forwarding",
                    "router-id": "0.0.0.0"
                },
                {
                    "instance-name": "__master.anon__",
                    "instance-state": "Active",
                    "instance-type": "forwarding",
                    "router-id": "0.0.0.0"
                },
                {
                    "instance-name": "mgmt_junos",
                    "instance-state": "Active",
                    "instance-type": "forwarding",
                    "router-id": "0.0.0.0"
                }
            ]
        }
    }

    def test_empty(self):
        self.device = Mock(**self.empty_output)
        obj = ShowRouteInstanceDetail(device=self.device)
        with self.assertRaises(SchemaEmptyParserError):
            parsed_output = obj.parse()

    def test_golden(self):
        self.device = Mock(**self.golden_output)
        obj = ShowRouteInstanceDetail(device=self.device)
        parsed_output = obj.parse()
        self.assertEqual(parsed_output, self.golden_parsed_output)

if __name__ == '__main__':
    unittest.main()